import Security
from PyObjCTools.TestSupport import TestCase


class TestMDS(TestCase):
    def test_unsuppported(self):
        self.assertFalse(hasattr(Security, "gGuidCssm"))
        self.assertFalse(hasattr(Security, "gGuidAppleFileDL"))
        self.assertFalse(hasattr(Security, "gGuidAppleCSP"))
        self.assertFalse(hasattr(Security, "gGuidAppleCSPDL"))
        self.assertFalse(hasattr(Security, "gGuidAppleX509CL"))
        self.assertFalse(hasattr(Security, "gGuidAppleX509TP"))
        self.assertFalse(hasattr(Security, "gGuidAppleLDAPDL"))
        self.assertFalse(hasattr(Security, "gGuidAppleDotMacTP"))
        self.assertFalse(hasattr(Security, "gGuidAppleSdCSPDL"))
        self.assertFalse(hasattr(Security, "gGuidAppleDotMacDL"))
        self.assertFalse(hasattr(Security, "CSSM_WORDID_KEYCHAIN_PROMPT"))
        self.assertFalse(hasattr(Security, "CSSM_WORDID_KEYCHAIN_LOCK"))
        self.assertFalse(hasattr(Security, "CSSM_WORDID_KEYCHAIN_CHANGE_LOCK"))
        self.assertFalse(hasattr(Security, "CSSM_WORDID_PROCESS"))
        self.assertFalse(hasattr(Security, "CSSM_WORDID__RESERVED_1"))
        self.assertFalse(hasattr(Security, "CSSM_WORDID_SYMMETRIC_KEY"))
        self.assertFalse(hasattr(Security, "CSSM_WORDID_SYSTEM"))
        self.assertFalse(hasattr(Security, "CSSM_WORDID_KEY"))
        self.assertFalse(hasattr(Security, "CSSM_WORDID_PIN"))
        self.assertFalse(hasattr(Security, "CSSM_WORDID_PREAUTH"))
        self.assertFalse(hasattr(Security, "CSSM_WORDID_PREAUTH_SOURCE"))
        self.assertFalse(hasattr(Security, "CSSM_WORDID_ASYMMETRIC_KEY"))
        self.assertFalse(hasattr(Security, "CSSM_WORDID_PARTITION"))
        self.assertFalse(hasattr(Security, "CSSM_WORDID_KEYBAG_KEY"))
        self.assertFalse(hasattr(Security, "CSSM_WORDID__FIRST_UNUSED"))
        self.assertFalse(hasattr(Security, "CSSM_ACL_SUBJECT_TYPE_KEYCHAIN_PROMPT"))
        self.assertFalse(hasattr(Security, "CSSM_ACL_SUBJECT_TYPE_PROCESS"))
        self.assertFalse(hasattr(Security, "CSSM_ACL_SUBJECT_TYPE_CODE_SIGNATURE"))
        self.assertFalse(hasattr(Security, "CSSM_ACL_SUBJECT_TYPE_COMMENT"))
        self.assertFalse(hasattr(Security, "CSSM_ACL_SUBJECT_TYPE_SYMMETRIC_KEY"))
        self.assertFalse(hasattr(Security, "CSSM_ACL_SUBJECT_TYPE_PREAUTH"))
        self.assertFalse(hasattr(Security, "CSSM_ACL_SUBJECT_TYPE_PREAUTH_SOURCE"))
        self.assertFalse(hasattr(Security, "CSSM_ACL_SUBJECT_TYPE_ASYMMETRIC_KEY"))
        self.assertFalse(hasattr(Security, "CSSM_ACL_SUBJECT_TYPE_PARTITION"))
        self.assertFalse(hasattr(Security, "CSSM_SAMPLE_TYPE_KEYCHAIN_PROMPT"))
        self.assertFalse(hasattr(Security, "CSSM_SAMPLE_TYPE_KEYCHAIN_LOCK"))
        self.assertFalse(hasattr(Security, "CSSM_SAMPLE_TYPE_KEYCHAIN_CHANGE_LOCK"))
        self.assertFalse(hasattr(Security, "CSSM_SAMPLE_TYPE_PROCESS"))
        self.assertFalse(hasattr(Security, "CSSM_SAMPLE_TYPE_COMMENT"))
        self.assertFalse(hasattr(Security, "CSSM_SAMPLE_TYPE_RETRY_ID"))
        self.assertFalse(hasattr(Security, "CSSM_SAMPLE_TYPE_SYMMETRIC_KEY"))
        self.assertFalse(hasattr(Security, "CSSM_SAMPLE_TYPE_PREAUTH"))
        self.assertFalse(hasattr(Security, "CSSM_SAMPLE_TYPE_ASYMMETRIC_KEY"))
        self.assertFalse(hasattr(Security, "CSSM_SAMPLE_TYPE_KEYBAG_KEY"))
        self.assertFalse(hasattr(Security, "CSSM_ACL_AUTHORIZATION_CHANGE_ACL"))
        self.assertFalse(hasattr(Security, "CSSM_ACL_AUTHORIZATION_CHANGE_OWNER"))
        self.assertFalse(hasattr(Security, "CSSM_ACL_AUTHORIZATION_PARTITION_ID"))
        self.assertFalse(hasattr(Security, "CSSM_ACL_AUTHORIZATION_INTEGRITY"))
        self.assertFalse(hasattr(Security, "CSSM_ACL_AUTHORIZATION_PREAUTH_BASE"))
        self.assertFalse(hasattr(Security, "CSSM_ACL_AUTHORIZATION_PREAUTH_END"))
        self.assertFalse(hasattr(Security, "CSSM_ACL_AUTHORIZATION_PREAUTH"))
        self.assertFalse(hasattr(Security, "CSSM_ACL_AUTHORIZATION_PREAUTH_SLOT"))
        self.assertFalse(hasattr(Security, "CSSM_ACL_AUTHORIZATION_IS_PREAUTH"))
        self.assertFalse(hasattr(Security, "CSSM_ACL_CODE_SIGNATURE_INVALID"))
        self.assertFalse(hasattr(Security, "CSSM_ACL_CODE_SIGNATURE_OSX"))
        self.assertFalse(hasattr(Security, "CSSM_ACL_MATCH_UID"))
        self.assertFalse(hasattr(Security, "CSSM_ACL_MATCH_GID"))
        self.assertFalse(hasattr(Security, "CSSM_ACL_MATCH_HONOR_ROOT"))
        self.assertFalse(hasattr(Security, "CSSM_ACL_MATCH_BITS"))
        self.assertFalse(hasattr(Security, "CSSM_ACL_PROCESS_SELECTOR_CURRENT_VERSION"))
        self.assertFalse(hasattr(Security, "CSSM_ACL_PROCESS_SUBJECT_SELECTOR"))
        self.assertFalse(hasattr(Security, "CSSM_ACL_KEYCHAIN_PROMPT_CURRENT_VERSION"))
        self.assertFalse(
            hasattr(Security, "CSSM_ACL_KEYCHAIN_PROMPT_REQUIRE_PASSPHRASE")
        )
        self.assertFalse(hasattr(Security, "CSSM_ACL_KEYCHAIN_PROMPT_UNSIGNED"))
        self.assertFalse(hasattr(Security, "CSSM_ACL_KEYCHAIN_PROMPT_UNSIGNED_ACT"))
        self.assertFalse(hasattr(Security, "CSSM_ACL_KEYCHAIN_PROMPT_INVALID"))
        self.assertFalse(hasattr(Security, "CSSM_ACL_KEYCHAIN_PROMPT_INVALID_ACT"))
        self.assertFalse(hasattr(Security, "CSSM_ACL_KEYCHAIN_PROMPT_SELECTOR"))
        self.assertFalse(hasattr(Security, "CSSM_ACL_PREAUTH_TRACKING_COUNT_MASK"))
        self.assertFalse(hasattr(Security, "CSSM_ACL_PREAUTH_TRACKING_BLOCKED"))
        self.assertFalse(hasattr(Security, "CSSM_ACL_PREAUTH_TRACKING_UNKNOWN"))
        self.assertFalse(hasattr(Security, "CSSM_ACL_PREAUTH_TRACKING_AUTHORIZED"))
        self.assertFalse(hasattr(Security, "CSSM_DB_ACCESS_RESET"))
        self.assertFalse(hasattr(Security, "CSSM_ALGID_APPLE_YARROW"))
        self.assertFalse(hasattr(Security, "CSSM_ALGID_AES"))
        self.assertFalse(hasattr(Security, "CSSM_ALGID_FEE"))
        self.assertFalse(hasattr(Security, "CSSM_ALGID_FEE_MD5"))
        self.assertFalse(hasattr(Security, "CSSM_ALGID_FEE_SHA1"))
        self.assertFalse(hasattr(Security, "CSSM_ALGID_FEED"))
        self.assertFalse(hasattr(Security, "CSSM_ALGID_FEEDEXP"))
        self.assertFalse(hasattr(Security, "CSSM_ALGID_ASC"))
        self.assertFalse(hasattr(Security, "CSSM_ALGID_SHA1HMAC_LEGACY"))
        self.assertFalse(hasattr(Security, "CSSM_ALGID_KEYCHAIN_KEY"))
        self.assertFalse(hasattr(Security, "CSSM_ALGID_PKCS12_PBE_ENCR"))
        self.assertFalse(hasattr(Security, "CSSM_ALGID_PKCS12_PBE_MAC"))
        self.assertFalse(hasattr(Security, "CSSM_ALGID_SECURE_PASSPHRASE"))
        self.assertFalse(hasattr(Security, "CSSM_ALGID_PBE_OPENSSL_MD5"))
        self.assertFalse(hasattr(Security, "CSSM_ALGID_SHA256"))
        self.assertFalse(hasattr(Security, "CSSM_ALGID_SHA384"))
        self.assertFalse(hasattr(Security, "CSSM_ALGID_SHA512"))
        self.assertFalse(hasattr(Security, "CSSM_ALGID_ENTROPY_DEFAULT"))
        self.assertFalse(hasattr(Security, "CSSM_ALGID_SHA224"))
        self.assertFalse(hasattr(Security, "CSSM_ALGID_SHA224WithRSA"))
        self.assertFalse(hasattr(Security, "CSSM_ALGID_SHA256WithRSA"))
        self.assertFalse(hasattr(Security, "CSSM_ALGID_SHA384WithRSA"))
        self.assertFalse(hasattr(Security, "CSSM_ALGID_SHA512WithRSA"))
        self.assertFalse(hasattr(Security, "CSSM_ALGID_OPENSSH1"))
        self.assertFalse(hasattr(Security, "CSSM_ALGID_SHA224WithECDSA"))
        self.assertFalse(hasattr(Security, "CSSM_ALGID_SHA256WithECDSA"))
        self.assertFalse(hasattr(Security, "CSSM_ALGID_SHA384WithECDSA"))
        self.assertFalse(hasattr(Security, "CSSM_ALGID_SHA512WithECDSA"))
        self.assertFalse(hasattr(Security, "CSSM_ALGID_ECDSA_SPECIFIED"))
        self.assertFalse(hasattr(Security, "CSSM_ALGID_ECDH_X963_KDF"))
        self.assertFalse(hasattr(Security, "CSSM_ALGID__FIRST_UNUSED"))
        self.assertFalse(hasattr(Security, "CSSM_PADDING_APPLE_SSLv2"))
        self.assertFalse(hasattr(Security, "CSSM_KEYBLOB_RAW_FORMAT_VENDOR_DEFINED"))
        self.assertFalse(hasattr(Security, "CSSM_KEYBLOB_RAW_FORMAT_X509"))
        self.assertFalse(hasattr(Security, "CSSM_KEYBLOB_RAW_FORMAT_OPENSSH"))
        self.assertFalse(hasattr(Security, "CSSM_KEYBLOB_RAW_FORMAT_OPENSSL"))
        self.assertFalse(hasattr(Security, "CSSM_KEYBLOB_RAW_FORMAT_OPENSSH2"))
        self.assertFalse(hasattr(Security, "CSSM_CUSTOM_COMMON_ERROR_EXTENT"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_NO_USER_INTERACTION"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_USER_CANCELED"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_SERVICE_NOT_AVAILABLE"))
        self.assertFalse(
            hasattr(Security, "CSSM_ERRCODE_INSUFFICIENT_CLIENT_IDENTIFICATION")
        )
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_DEVICE_RESET"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_DEVICE_FAILED"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_IN_DARK_WAKE"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSSM_NO_USER_INTERACTION"))
        self.assertFalse(hasattr(Security, "CSSMERR_AC_NO_USER_INTERACTION"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_NO_USER_INTERACTION"))
        self.assertFalse(hasattr(Security, "CSSMERR_CL_NO_USER_INTERACTION"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_NO_USER_INTERACTION"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_NO_USER_INTERACTION"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSSM_USER_CANCELED"))
        self.assertFalse(hasattr(Security, "CSSMERR_AC_USER_CANCELED"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_USER_CANCELED"))
        self.assertFalse(hasattr(Security, "CSSMERR_CL_USER_CANCELED"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_USER_CANCELED"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_USER_CANCELED"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSSM_SERVICE_NOT_AVAILABLE"))
        self.assertFalse(hasattr(Security, "CSSMERR_AC_SERVICE_NOT_AVAILABLE"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_SERVICE_NOT_AVAILABLE"))
        self.assertFalse(hasattr(Security, "CSSMERR_CL_SERVICE_NOT_AVAILABLE"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_SERVICE_NOT_AVAILABLE"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_SERVICE_NOT_AVAILABLE"))
        self.assertFalse(
            hasattr(Security, "CSSMERR_CSSM_INSUFFICIENT_CLIENT_IDENTIFICATION")
        )
        self.assertFalse(
            hasattr(Security, "CSSMERR_AC_INSUFFICIENT_CLIENT_IDENTIFICATION")
        )
        self.assertFalse(
            hasattr(Security, "CSSMERR_CSP_INSUFFICIENT_CLIENT_IDENTIFICATION")
        )
        self.assertFalse(
            hasattr(Security, "CSSMERR_CL_INSUFFICIENT_CLIENT_IDENTIFICATION")
        )
        self.assertFalse(
            hasattr(Security, "CSSMERR_DL_INSUFFICIENT_CLIENT_IDENTIFICATION")
        )
        self.assertFalse(
            hasattr(Security, "CSSMERR_TP_INSUFFICIENT_CLIENT_IDENTIFICATION")
        )
        self.assertFalse(hasattr(Security, "CSSMERR_CSSM_DEVICE_RESET"))
        self.assertFalse(hasattr(Security, "CSSMERR_AC_DEVICE_RESET"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_DEVICE_RESET"))
        self.assertFalse(hasattr(Security, "CSSMERR_CL_DEVICE_RESET"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_DEVICE_RESET"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_DEVICE_RESET"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSSM_DEVICE_FAILED"))
        self.assertFalse(hasattr(Security, "CSSMERR_AC_DEVICE_FAILED"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_DEVICE_FAILED"))
        self.assertFalse(hasattr(Security, "CSSMERR_CL_DEVICE_FAILED"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_DEVICE_FAILED"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_DEVICE_FAILED"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSSM_IN_DARK_WAKE"))
        self.assertFalse(hasattr(Security, "CSSMERR_AC_IN_DARK_WAKE"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_IN_DARK_WAKE"))
        self.assertFalse(hasattr(Security, "CSSMERR_CL_IN_DARK_WAKE"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_IN_DARK_WAKE"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_IN_DARK_WAKE"))
        self.assertFalse(
            hasattr(Security, "CSSMERR_CSP_APPLE_ADD_APPLICATION_ACL_SUBJECT")
        )
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_APPLE_PUBLIC_KEY_INCOMPLETE"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_APPLE_SIGNATURE_MISMATCH"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_APPLE_INVALID_KEY_START_DATE"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_APPLE_INVALID_KEY_END_DATE"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSPDL_APPLE_DL_CONVERSION_ERROR"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_APPLE_SSLv2_ROLLBACK"))
        self.assertFalse(hasattr(Security, "CSSM_DL_DB_RECORD_GENERIC_PASSWORD"))
        self.assertFalse(hasattr(Security, "CSSM_DL_DB_RECORD_INTERNET_PASSWORD"))
        self.assertFalse(hasattr(Security, "CSSM_DL_DB_RECORD_APPLESHARE_PASSWORD"))
        self.assertFalse(hasattr(Security, "CSSM_DL_DB_RECORD_X509_CERTIFICATE"))
        self.assertFalse(hasattr(Security, "CSSM_DL_DB_RECORD_USER_TRUST"))
        self.assertFalse(hasattr(Security, "CSSM_DL_DB_RECORD_X509_CRL"))
        self.assertFalse(hasattr(Security, "CSSM_DL_DB_RECORD_UNLOCK_REFERRAL"))
        self.assertFalse(hasattr(Security, "CSSM_DL_DB_RECORD_EXTENDED_ATTRIBUTE"))
        self.assertFalse(hasattr(Security, "CSSM_DL_DB_RECORD_METADATA"))
        self.assertFalse(hasattr(Security, "CSSM_APPLEFILEDL_TOGGLE_AUTOCOMMIT"))
        self.assertFalse(hasattr(Security, "CSSM_APPLEFILEDL_COMMIT"))
        self.assertFalse(hasattr(Security, "CSSM_APPLEFILEDL_ROLLBACK"))
        self.assertFalse(hasattr(Security, "CSSM_APPLEFILEDL_TAKE_FILE_LOCK"))
        self.assertFalse(hasattr(Security, "CSSM_APPLEFILEDL_MAKE_BACKUP"))
        self.assertFalse(hasattr(Security, "CSSM_APPLEFILEDL_MAKE_COPY"))
        self.assertFalse(hasattr(Security, "CSSM_APPLEFILEDL_DELETE_FILE"))
        self.assertFalse(hasattr(Security, "CSSM_APPLE_UNLOCK_TYPE_KEY_DIRECT"))
        self.assertFalse(hasattr(Security, "CSSM_APPLE_UNLOCK_TYPE_WRAPPED_PRIVATE"))
        self.assertFalse(hasattr(Security, "CSSM_APPLE_UNLOCK_TYPE_KEYBAG"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLEDL_INVALID_OPEN_PARAMETERS"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLEDL_DISK_FULL"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLEDL_QUOTA_EXCEEDED"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLEDL_FILE_TOO_BIG"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLEDL_INVALID_DATABASE_BLOB"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLEDL_INVALID_KEY_BLOB"))
        self.assertFalse(
            hasattr(Security, "CSSMERR_APPLEDL_INCOMPATIBLE_DATABASE_BLOB")
        )
        self.assertFalse(hasattr(Security, "CSSMERR_APPLEDL_INCOMPATIBLE_KEY_BLOB"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_HOSTNAME_MISMATCH"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_UNKNOWN_CRITICAL_EXTEN"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_NO_BASIC_CONSTRAINTS"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_INVALID_CA"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_INVALID_AUTHORITY_ID"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_INVALID_SUBJECT_ID"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_INVALID_KEY_USAGE"))
        self.assertFalse(
            hasattr(Security, "CSSMERR_APPLETP_INVALID_EXTENDED_KEY_USAGE")
        )
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_INVALID_ID_LINKAGE"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_PATH_LEN_CONSTRAINT"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_INVALID_ROOT"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_CRL_EXPIRED"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_CRL_NOT_FOUND"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_CRL_SERVER_DOWN"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_CRL_BAD_URI"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_UNKNOWN_CERT_EXTEN"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_UNKNOWN_CRL_EXTEN"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_CRL_NOT_TRUSTED"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_CRL_INVALID_ANCHOR_CERT"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_CRL_POLICY_FAIL"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_IDP_FAIL"))
        self.assertFalse(
            hasattr(Security, "CSSMERR_APPLETP_CERT_NOT_FOUND_FROM_ISSUER")
        )
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_BAD_CERT_FROM_ISSUER"))
        self.assertFalse(
            hasattr(Security, "CSSMERR_APPLETP_SMIME_EMAIL_ADDRS_NOT_FOUND")
        )
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_SMIME_BAD_EXT_KEY_USE"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_SMIME_BAD_KEY_USE"))
        self.assertFalse(
            hasattr(Security, "CSSMERR_APPLETP_SMIME_KEYUSAGE_NOT_CRITICAL")
        )
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_SMIME_NO_EMAIL_ADDRS"))
        self.assertFalse(
            hasattr(Security, "CSSMERR_APPLETP_SMIME_SUBJ_ALT_NAME_NOT_CRIT")
        )
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_SSL_BAD_EXT_KEY_USE"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_OCSP_BAD_RESPONSE"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_OCSP_BAD_REQUEST"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_OCSP_UNAVAILABLE"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_OCSP_STATUS_UNRECOGNIZED"))
        self.assertFalse(
            hasattr(Security, "CSSMERR_APPLETP_INCOMPLETE_REVOCATION_CHECK")
        )
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_NETWORK_FAILURE"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_OCSP_NOT_TRUSTED"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_OCSP_INVALID_ANCHOR_CERT"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_OCSP_SIG_ERROR"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_OCSP_NO_SIGNER"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_OCSP_RESP_MALFORMED_REQ"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_OCSP_RESP_INTERNAL_ERR"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_OCSP_RESP_TRY_LATER"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_OCSP_RESP_SIG_REQUIRED"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_OCSP_RESP_UNAUTHORIZED"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_OCSP_NONCE_MISMATCH"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_CS_BAD_CERT_CHAIN_LENGTH"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_CS_NO_BASIC_CONSTRAINTS"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_CS_BAD_PATH_LENGTH"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_CS_NO_EXTENDED_KEY_USAGE"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_CODE_SIGN_DEVELOPMENT"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_RS_BAD_CERT_CHAIN_LENGTH"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_RS_BAD_EXTENDED_KEY_USAGE"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_TRUST_SETTING_DENY"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_INVALID_EMPTY_SUBJECT"))
        self.assertFalse(
            hasattr(Security, "CSSMERR_APPLETP_UNKNOWN_QUAL_CERT_STATEMENT")
        )
        self.assertFalse(
            hasattr(Security, "CSSMERR_APPLETP_MISSING_REQUIRED_EXTENSION")
        )
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_EXT_KEYUSAGE_NOT_CRITICAL"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_IDENTIFIER_MISSING"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_CA_PIN_MISMATCH"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLE_DOTMAC_REQ_QUEUED"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLE_DOTMAC_REQ_REDIRECT"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLE_DOTMAC_REQ_SERVER_ERR"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLE_DOTMAC_REQ_SERVER_PARAM"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLE_DOTMAC_REQ_SERVER_AUTH"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLE_DOTMAC_REQ_SERVER_UNIMPL"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLE_DOTMAC_REQ_SERVER_NOT_AVAIL"))
        self.assertFalse(
            hasattr(Security, "CSSMERR_APPLE_DOTMAC_REQ_SERVER_ALREADY_EXIST")
        )
        self.assertFalse(
            hasattr(Security, "CSSMERR_APPLE_DOTMAC_REQ_SERVER_SERVICE_ERROR")
        )
        self.assertFalse(hasattr(Security, "CSSMERR_APPLE_DOTMAC_REQ_IS_PENDING"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLE_DOTMAC_NO_REQ_PENDING"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLE_DOTMAC_CSR_VERIFY_FAIL"))
        self.assertFalse(
            hasattr(Security, "CSSMERR_APPLE_DOTMAC_FAILED_CONSISTENCY_CHECK")
        )
        self.assertFalse(hasattr(Security, "CSSM_APPLEDL_OPEN_PARAMETERS_VERSION"))
        self.assertFalse(hasattr(Security, "kCSSM_APPLEDL_MASK_MODE"))
        self.assertFalse(hasattr(Security, "CSSM_APPLEDL_OPEN_PARAMETERS"))
        self.assertFalse(hasattr(Security, "CSSM_APPLECSPDL_DB_LOCK"))
        self.assertFalse(hasattr(Security, "CSSM_APPLECSPDL_DB_UNLOCK"))
        self.assertFalse(hasattr(Security, "CSSM_APPLECSPDL_DB_GET_SETTINGS"))
        self.assertFalse(hasattr(Security, "CSSM_APPLECSPDL_DB_SET_SETTINGS"))
        self.assertFalse(hasattr(Security, "CSSM_APPLECSPDL_DB_IS_LOCKED"))
        self.assertFalse(hasattr(Security, "CSSM_APPLECSPDL_DB_CHANGE_PASSWORD"))
        self.assertFalse(hasattr(Security, "CSSM_APPLECSPDL_DB_GET_HANDLE"))
        self.assertFalse(hasattr(Security, "CSSM_APPLESCPDL_CSP_GET_KEYHANDLE"))
        self.assertFalse(hasattr(Security, "CSSM_APPLE_PRIVATE_CSPDL_CODE_8"))
        self.assertFalse(hasattr(Security, "CSSM_APPLE_PRIVATE_CSPDL_CODE_9"))
        self.assertFalse(hasattr(Security, "CSSM_APPLE_PRIVATE_CSPDL_CODE_10"))
        self.assertFalse(hasattr(Security, "CSSM_APPLE_PRIVATE_CSPDL_CODE_11"))
        self.assertFalse(hasattr(Security, "CSSM_APPLE_PRIVATE_CSPDL_CODE_12"))
        self.assertFalse(hasattr(Security, "CSSM_APPLE_PRIVATE_CSPDL_CODE_13"))
        self.assertFalse(hasattr(Security, "CSSM_APPLE_PRIVATE_CSPDL_CODE_14"))
        self.assertFalse(hasattr(Security, "CSSM_APPLE_PRIVATE_CSPDL_CODE_15"))
        self.assertFalse(hasattr(Security, "CSSM_APPLE_PRIVATE_CSPDL_CODE_16"))
        self.assertFalse(hasattr(Security, "CSSM_APPLE_PRIVATE_CSPDL_CODE_17"))
        self.assertFalse(hasattr(Security, "CSSM_APPLE_PRIVATE_CSPDL_CODE_18"))
        self.assertFalse(hasattr(Security, "CSSM_APPLE_PRIVATE_CSPDL_CODE_19"))
        self.assertFalse(hasattr(Security, "CSSM_APPLE_PRIVATE_CSPDL_CODE_20"))
        self.assertFalse(hasattr(Security, "CSSM_APPLE_PRIVATE_CSPDL_CODE_21"))
        self.assertFalse(hasattr(Security, "CSSM_APPLE_PRIVATE_CSPDL_CODE_22"))
        self.assertFalse(hasattr(Security, "CSSM_APPLE_PRIVATE_CSPDL_CODE_23"))
        self.assertFalse(hasattr(Security, "CSSM_APPLE_PRIVATE_CSPDL_CODE_24"))
        self.assertFalse(hasattr(Security, "CSSM_APPLE_PRIVATE_CSPDL_CODE_25"))
        self.assertFalse(hasattr(Security, "CSSM_APPLE_PRIVATE_CSPDL_CODE_26"))
        self.assertFalse(hasattr(Security, "CSSM_APPLE_PRIVATE_CSPDL_CODE_27"))
        self.assertFalse(hasattr(Security, "CSSM_APPLECSP_KEYDIGEST"))
        self.assertFalse(hasattr(Security, "CSSM_APPLECSPDL_DB_SETTINGS_PARAMETERS"))
        self.assertFalse(hasattr(Security, "CSSM_APPLECSPDL_DB_IS_LOCKED_PARAMETERS"))
        self.assertFalse(
            hasattr(Security, "CSSM_APPLECSPDL_DB_CHANGE_PASSWORD_PARAMETERS")
        )
        self.assertFalse(hasattr(Security, "CSSM_KEYBLOB_WRAPPED_FORMAT_APPLE_CUSTOM"))
        self.assertFalse(hasattr(Security, "CSSM_KEYBLOB_WRAPPED_FORMAT_OPENSSL"))
        self.assertFalse(hasattr(Security, "CSSM_KEYBLOB_WRAPPED_FORMAT_OPENSSH1"))
        self.assertFalse(hasattr(Security, "CSSM_ATTRIBUTE_VENDOR_DEFINED"))
        self.assertFalse(hasattr(Security, "CSSM_ATTRIBUTE_PUBLIC_KEY"))
        self.assertFalse(hasattr(Security, "CSSM_ATTRIBUTE_FEE_PRIME_TYPE"))
        self.assertFalse(hasattr(Security, "CSSM_ATTRIBUTE_FEE_CURVE_TYPE"))
        self.assertFalse(hasattr(Security, "CSSM_ATTRIBUTE_ASC_OPTIMIZATION"))
        self.assertFalse(hasattr(Security, "CSSM_ATTRIBUTE_RSA_BLINDING"))
        self.assertFalse(hasattr(Security, "CSSM_ATTRIBUTE_PARAM_KEY"))
        self.assertFalse(hasattr(Security, "CSSM_ATTRIBUTE_PROMPT"))
        self.assertFalse(hasattr(Security, "CSSM_ATTRIBUTE_ALERT_TITLE"))
        self.assertFalse(hasattr(Security, "CSSM_ATTRIBUTE_VERIFY_PASSPHRASE"))
        self.assertFalse(hasattr(Security, "CSSM_FEE_PRIME_TYPE_DEFAULT"))
        self.assertFalse(hasattr(Security, "CSSM_FEE_PRIME_TYPE_MERSENNE"))
        self.assertFalse(hasattr(Security, "CSSM_FEE_PRIME_TYPE_FEE"))
        self.assertFalse(hasattr(Security, "CSSM_FEE_PRIME_TYPE_GENERAL"))
        self.assertFalse(hasattr(Security, "CSSM_FEE_CURVE_TYPE_DEFAULT"))
        self.assertFalse(hasattr(Security, "CSSM_FEE_CURVE_TYPE_MONTGOMERY"))
        self.assertFalse(hasattr(Security, "CSSM_FEE_CURVE_TYPE_WEIERSTRASS"))
        self.assertFalse(hasattr(Security, "CSSM_FEE_CURVE_TYPE_ANSI_X9_62"))
        self.assertFalse(hasattr(Security, "CSSM_ASC_OPTIMIZE_DEFAULT"))
        self.assertFalse(hasattr(Security, "CSSM_ASC_OPTIMIZE_SIZE"))
        self.assertFalse(hasattr(Security, "CSSM_ASC_OPTIMIZE_SECURITY"))
        self.assertFalse(hasattr(Security, "CSSM_ASC_OPTIMIZE_TIME"))
        self.assertFalse(hasattr(Security, "CSSM_ASC_OPTIMIZE_TIME_SIZE"))
        self.assertFalse(hasattr(Security, "CSSM_ASC_OPTIMIZE_ASCII"))
        self.assertFalse(hasattr(Security, "CSSM_KEYATTR_PARTIAL"))
        self.assertFalse(hasattr(Security, "CSSM_KEYATTR_PUBLIC_KEY_ENCRYPT"))
        self.assertFalse(hasattr(Security, "CSSM_APPLE_TP_NAME_OID"))
        self.assertFalse(hasattr(Security, "CSSM_APPLE_TP_CERT_REQUEST"))
        self.assertFalse(hasattr(Security, "CSSM_APPLE_TP_SSL_OPTS_VERSION"))
        self.assertFalse(hasattr(Security, "CSSM_APPLE_TP_SSL_CLIENT"))
        self.assertFalse(hasattr(Security, "CSSM_APPLE_TP_SSL_OPTIONS"))
        self.assertFalse(hasattr(Security, "CSSM_APPLE_TP_CRL_OPTS_VERSION"))
        self.assertFalse(hasattr(Security, "CSSM_TP_ACTION_REQUIRE_CRL_PER_CERT"))
        self.assertFalse(hasattr(Security, "CSSM_TP_ACTION_FETCH_CRL_FROM_NET"))
        self.assertFalse(hasattr(Security, "CSSM_TP_ACTION_CRL_SUFFICIENT"))
        self.assertFalse(hasattr(Security, "CSSM_TP_ACTION_REQUIRE_CRL_IF_PRESENT"))
        self.assertFalse(hasattr(Security, "CSSM_APPLE_TP_CRL_OPTIONS"))
        self.assertFalse(hasattr(Security, "CSSM_APPLE_TP_SMIME_OPTS_VERSION"))
        self.assertFalse(hasattr(Security, "CSSM_APPLE_TP_SMIME_OPTIONS"))
        self.assertFalse(hasattr(Security, "CSSM_TP_ACTION_ALLOW_EXPIRED"))
        self.assertFalse(hasattr(Security, "CSSM_TP_ACTION_LEAF_IS_CA"))
        self.assertFalse(hasattr(Security, "CSSM_TP_ACTION_FETCH_CERT_FROM_NET"))
        self.assertFalse(hasattr(Security, "CSSM_TP_ACTION_ALLOW_EXPIRED_ROOT"))
        self.assertFalse(hasattr(Security, "CSSM_TP_ACTION_REQUIRE_REV_PER_CERT"))
        self.assertFalse(hasattr(Security, "CSSM_TP_ACTION_TRUST_SETTINGS"))
        self.assertFalse(hasattr(Security, "CSSM_TP_ACTION_IMPLICIT_ANCHORS"))
        self.assertFalse(hasattr(Security, "CSSM_APPLE_TP_ACTION_VERSION"))
        self.assertFalse(hasattr(Security, "CSSM_APPLE_TP_ACTION_DATA"))
        self.assertFalse(hasattr(Security, "CSSM_CERT_STATUS_EXPIRED"))
        self.assertFalse(hasattr(Security, "CSSM_CERT_STATUS_NOT_VALID_YET"))
        self.assertFalse(hasattr(Security, "CSSM_CERT_STATUS_IS_IN_INPUT_CERTS"))
        self.assertFalse(hasattr(Security, "CSSM_CERT_STATUS_IS_IN_ANCHORS"))
        self.assertFalse(hasattr(Security, "CSSM_CERT_STATUS_IS_ROOT"))
        self.assertFalse(hasattr(Security, "CSSM_CERT_STATUS_IS_FROM_NET"))
        self.assertFalse(
            hasattr(Security, "CSSM_CERT_STATUS_TRUST_SETTINGS_FOUND_USER")
        )
        self.assertFalse(
            hasattr(Security, "CSSM_CERT_STATUS_TRUST_SETTINGS_FOUND_ADMIN")
        )
        self.assertFalse(
            hasattr(Security, "CSSM_CERT_STATUS_TRUST_SETTINGS_FOUND_SYSTEM")
        )
        self.assertFalse(hasattr(Security, "CSSM_CERT_STATUS_TRUST_SETTINGS_TRUST"))
        self.assertFalse(hasattr(Security, "CSSM_CERT_STATUS_TRUST_SETTINGS_DENY"))
        self.assertFalse(
            hasattr(Security, "CSSM_CERT_STATUS_TRUST_SETTINGS_IGNORED_ERROR")
        )
        self.assertFalse(hasattr(Security, "CSSM_TP_APPLE_EVIDENCE_INFO"))
        self.assertFalse(hasattr(Security, "CSSM_TP_APPLE_EVIDENCE_VERSION"))
        self.assertFalse(hasattr(Security, "CSSM_TP_APPLE_EVIDENCE_HEADER"))
        self.assertFalse(hasattr(Security, "CSSM_EVIDENCE_FORM_APPLE_CUSTOM"))
        self.assertFalse(hasattr(Security, "CSSM_EVIDENCE_FORM_APPLE_HEADER"))
        self.assertFalse(hasattr(Security, "CSSM_EVIDENCE_FORM_APPLE_CERTGROUP"))
        self.assertFalse(hasattr(Security, "CSSM_EVIDENCE_FORM_APPLE_CERT_INFO"))
        self.assertFalse(hasattr(Security, "CSSM_APPLEX509CL_OBTAIN_CSR"))
        self.assertFalse(hasattr(Security, "CSSM_APPLEX509CL_VERIFY_CSR"))
        self.assertFalse(hasattr(Security, "CSSM_APPLE_CL_CSR_REQUEST"))
        self.assertFalse(hasattr(Security, "CSSM_APPLE_CRL_END_OF_TIME"))
        self.assertFalse(hasattr(Security, "kKeychainSuffix"))
        self.assertFalse(hasattr(Security, "kKeychainDbSuffix"))
        self.assertFalse(hasattr(Security, "kSystemKeychainName"))
        self.assertFalse(hasattr(Security, "kSystemKeychainDir"))
        self.assertFalse(hasattr(Security, "kSystemKeychainPath"))
        self.assertFalse(hasattr(Security, "kSystemUnlockFile"))
        self.assertFalse(hasattr(Security, "CSSM_APPLE_ACL_TAG_PARTITION_ID"))
        self.assertFalse(hasattr(Security, "CSSM_APPLE_ACL_TAG_INTEGRITY"))
        self.assertFalse(hasattr(Security, "cssmPerror"))
        self.assertFalse(hasattr(Security, "cssmOidToAlg"))
        self.assertFalse(hasattr(Security, "cssmAlgToOid"))
        self.assertFalse(hasattr(Security, "errSecErrnoBase"))
        self.assertFalse(hasattr(Security, "errSecErrnoLimit"))
        self.assertFalse(hasattr(Security, "CSSMERR_APPLETP_LEAF_PIN_MISMATCH"))



import Security
from PyObjCTools.TestSupport import TestCase


class Testoidscert(TestCase):
    def test_unsuppported(self):
        self.assertFalse(hasattr(Security, "INTEL_X509V3_CERT_R08"))
        self.assertFalse(hasattr(Security, "INTEL_X509V3_CERT_R08_LENGTH"))
        self.assertFalse(hasattr(Security, "INTEL_X509V3_CERT_PRIVATE_EXTENSIONS"))
        self.assertFalse(
            hasattr(Security, "INTEL_X509V3_CERT_PRIVATE_EXTENSIONS_LENGTH")
        )
        self.assertFalse(hasattr(Security, "INTEL_X509V3_SIGN_R08"))
        self.assertFalse(hasattr(Security, "INTEL_X509V3_SIGN_R08_LENGTH"))
        self.assertFalse(hasattr(Security, "INTEL_X509_C_DATATYPE"))
        self.assertFalse(hasattr(Security, "INTEL_X509_LDAPSTRING_DATATYPE"))
        self.assertFalse(hasattr(Security, "CSSMOID_X509V3SignedCertificate"))
        self.assertFalse(hasattr(Security, "CSSMOID_X509V3SignedCertificateCStruct"))
        self.assertFalse(hasattr(Security, "CSSMOID_X509V3Certificate"))
        self.assertFalse(hasattr(Security, "CSSMOID_X509V3CertificateCStruct"))
        self.assertFalse(hasattr(Security, "CSSMOID_X509V1Version"))
        self.assertFalse(hasattr(Security, "CSSMOID_X509V1SerialNumber"))
        self.assertFalse(hasattr(Security, "CSSMOID_X509V1IssuerName"))
        self.assertFalse(hasattr(Security, "CSSMOID_X509V1IssuerNameStd"))
        self.assertFalse(hasattr(Security, "CSSMOID_X509V1IssuerNameCStruct"))
        self.assertFalse(hasattr(Security, "CSSMOID_X509V1IssuerNameLDAP"))
        self.assertFalse(hasattr(Security, "CSSMOID_X509V1ValidityNotBefore"))
        self.assertFalse(hasattr(Security, "CSSMOID_X509V1ValidityNotAfter"))
        self.assertFalse(hasattr(Security, "CSSMOID_X509V1SubjectName"))
        self.assertFalse(hasattr(Security, "CSSMOID_X509V1SubjectNameStd"))
        self.assertFalse(hasattr(Security, "CSSMOID_X509V1SubjectNameCStruct"))
        self.assertFalse(hasattr(Security, "CSSMOID_X509V1SubjectNameLDAP"))
        self.assertFalse(hasattr(Security, "CSSMOID_CSSMKeyStruct"))
        self.assertFalse(hasattr(Security, "CSSMOID_X509V1SubjectPublicKeyCStruct"))
        self.assertFalse(hasattr(Security, "CSSMOID_X509V1SubjectPublicKeyAlgorithm"))
        self.assertFalse(
            hasattr(Security, "CSSMOID_X509V1SubjectPublicKeyAlgorithmParameters")
        )
        self.assertFalse(hasattr(Security, "CSSMOID_X509V1SubjectPublicKey"))
        self.assertFalse(hasattr(Security, "CSSMOID_X509V1CertificateIssuerUniqueId"))
        self.assertFalse(hasattr(Security, "CSSMOID_X509V1CertificateSubjectUniqueId"))
        self.assertFalse(hasattr(Security, "CSSMOID_X509V3CertificateExtensionsStruct"))
        self.assertFalse(
            hasattr(Security, "CSSMOID_X509V3CertificateExtensionsCStruct")
        )
        self.assertFalse(
            hasattr(Security, "CSSMOID_X509V3CertificateNumberOfExtensions")
        )
        self.assertFalse(hasattr(Security, "CSSMOID_X509V3CertificateExtensionStruct"))
        self.assertFalse(hasattr(Security, "CSSMOID_X509V3CertificateExtensionCStruct"))
        self.assertFalse(hasattr(Security, "CSSMOID_X509V3CertificateExtensionId"))
        self.assertFalse(
            hasattr(Security, "CSSMOID_X509V3CertificateExtensionCritical")
        )
        self.assertFalse(hasattr(Security, "CSSMOID_X509V3CertificateExtensionType"))
        self.assertFalse(hasattr(Security, "CSSMOID_X509V3CertificateExtensionValue"))
        self.assertFalse(hasattr(Security, "CSSMOID_X509V1SignatureStruct"))
        self.assertFalse(hasattr(Security, "CSSMOID_X509V1SignatureCStruct"))
        self.assertFalse(hasattr(Security, "CSSMOID_X509V1SignatureAlgorithm"))
        self.assertFalse(hasattr(Security, "CSSMOID_X509V1SignatureAlgorithmTBS"))
        self.assertFalse(
            hasattr(Security, "CSSMOID_X509V1SignatureAlgorithmParameters")
        )
        self.assertFalse(hasattr(Security, "CSSMOID_X509V1Signature"))
        self.assertFalse(hasattr(Security, "CSSMOID_SubjectSignatureBitmap"))
        self.assertFalse(hasattr(Security, "CSSMOID_SubjectPicture"))
        self.assertFalse(hasattr(Security, "CSSMOID_SubjectEmailAddress"))
        self.assertFalse(hasattr(Security, "CSSMOID_UseExemptions"))
        self.assertFalse(hasattr(Security, "CSSMOID_SubjectDirectoryAttributes"))
        self.assertFalse(hasattr(Security, "CSSMOID_SubjectKeyIdentifier"))
        self.assertFalse(hasattr(Security, "CSSMOID_KeyUsage"))
        self.assertFalse(hasattr(Security, "CSSMOID_PrivateKeyUsagePeriod"))
        self.assertFalse(hasattr(Security, "CSSMOID_SubjectAltName"))
        self.assertFalse(hasattr(Security, "CSSMOID_IssuerAltName"))
        self.assertFalse(hasattr(Security, "CSSMOID_BasicConstraints"))
        self.assertFalse(hasattr(Security, "CSSMOID_CrlNumber"))
        self.assertFalse(hasattr(Security, "CSSMOID_CrlReason"))
        self.assertFalse(hasattr(Security, "CSSMOID_HoldInstructionCode"))
        self.assertFalse(hasattr(Security, "CSSMOID_InvalidityDate"))
        self.assertFalse(hasattr(Security, "CSSMOID_DeltaCrlIndicator"))
        self.assertFalse(hasattr(Security, "CSSMOID_IssuingDistributionPoint"))
        self.assertFalse(hasattr(Security, "CSSMOID_IssuingDistributionPoints"))
        self.assertFalse(hasattr(Security, "CSSMOID_CertIssuer"))
        self.assertFalse(hasattr(Security, "CSSMOID_NameConstraints"))
        self.assertFalse(hasattr(Security, "CSSMOID_CrlDistributionPoints"))
        self.assertFalse(hasattr(Security, "CSSMOID_CertificatePolicies"))
        self.assertFalse(hasattr(Security, "CSSMOID_PolicyMappings"))
        self.assertFalse(hasattr(Security, "CSSMOID_PolicyConstraints"))
        self.assertFalse(hasattr(Security, "CSSMOID_AuthorityKeyIdentifier"))
        self.assertFalse(hasattr(Security, "CSSMOID_ExtendedKeyUsage"))
        self.assertFalse(hasattr(Security, "CSSMOID_InhibitAnyPolicy"))
        self.assertFalse(hasattr(Security, "CSSMOID_AuthorityInfoAccess"))
        self.assertFalse(hasattr(Security, "CSSMOID_BiometricInfo"))
        self.assertFalse(hasattr(Security, "CSSMOID_QC_Statements"))
        self.assertFalse(hasattr(Security, "CSSMOID_SubjectInfoAccess"))
        self.assertFalse(hasattr(Security, "CSSMOID_ExtendedKeyUsageAny"))
        self.assertFalse(hasattr(Security, "CSSMOID_ServerAuth"))
        self.assertFalse(hasattr(Security, "CSSMOID_ClientAuth"))
        self.assertFalse(hasattr(Security, "CSSMOID_ExtendedUseCodeSigning"))
        self.assertFalse(hasattr(Security, "CSSMOID_EmailProtection"))
        self.assertFalse(hasattr(Security, "CSSMOID_TimeStamping"))
        self.assertFalse(hasattr(Security, "CSSMOID_OCSPSigning"))
        self.assertFalse(hasattr(Security, "CSSMOID_KERBv5_PKINIT_KP_CLIENT_AUTH"))
        self.assertFalse(hasattr(Security, "CSSMOID_KERBv5_PKINIT_KP_KDC"))
        self.assertFalse(hasattr(Security, "CSSMOID_EKU_IPSec"))
        self.assertFalse(hasattr(Security, "CSSMOID_DOTMAC_CERT_EXTENSION"))
        self.assertFalse(hasattr(Security, "CSSMOID_DOTMAC_CERT_IDENTITY"))
        self.assertFalse(hasattr(Security, "CSSMOID_DOTMAC_CERT_EMAIL_SIGN"))
        self.assertFalse(hasattr(Security, "CSSMOID_DOTMAC_CERT_EMAIL_ENCRYPT"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_CERT_POLICY"))
        self.assertFalse(hasattr(Security, "CSSMOID_DOTMAC_CERT_POLICY"))
        self.assertFalse(hasattr(Security, "CSSMOID_ADC_CERT_POLICY"))
        self.assertFalse(hasattr(Security, "CSSMOID_MACAPPSTORE_CERT_POLICY"))
        self.assertFalse(hasattr(Security, "CSSMOID_MACAPPSTORE_RECEIPT_CERT_POLICY"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLEID_CERT_POLICY"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLEID_SHARING_CERT_POLICY"))
        self.assertFalse(hasattr(Security, "CSSMOID_MOBILE_STORE_SIGNING_POLICY"))
        self.assertFalse(hasattr(Security, "CSSMOID_TEST_MOBILE_STORE_SIGNING_POLICY"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_EKU_CODE_SIGNING"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_EKU_CODE_SIGNING_DEV"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_EKU_RESOURCE_SIGNING"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_EKU_ICHAT_SIGNING"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_EKU_ICHAT_ENCRYPTION"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_EKU_SYSTEM_IDENTITY"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_EKU_PASSBOOK_SIGNING"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_EKU_PROFILE_SIGNING"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_EKU_QA_PROFILE_SIGNING"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_EXTENSION"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_EXTENSION_CODE_SIGNING"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_EXTENSION_APPLE_SIGNING"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_EXTENSION_ADC_DEV_SIGNING"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_EXTENSION_ADC_APPLE_SIGNING"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_EXTENSION_PASSBOOK_SIGNING"))
        self.assertFalse(
            hasattr(Security, "CSSMOID_APPLE_EXTENSION_MACAPPSTORE_RECEIPT")
        )
        self.assertFalse(
            hasattr(Security, "CSSMOID_APPLE_EXTENSION_INTERMEDIATE_MARKER")
        )
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_EXTENSION_WWDR_INTERMEDIATE"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_EXTENSION_ITMS_INTERMEDIATE"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_EXTENSION_AAI_INTERMEDIATE"))
        self.assertFalse(
            hasattr(Security, "CSSMOID_APPLE_EXTENSION_APPLEID_INTERMEDIATE")
        )
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_EXTENSION_APPLEID_SHARING"))
        self.assertFalse(
            hasattr(Security, "CSSMOID_APPLE_EXTENSION_SYSINT2_INTERMEDIATE")
        )
        self.assertFalse(
            hasattr(Security, "CSSMOID_APPLE_EXTENSION_DEVELOPER_AUTHENTICATION")
        )
        self.assertFalse(
            hasattr(Security, "CSSMOID_APPLE_EXTENSION_SERVER_AUTHENTICATION")
        )
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_EXTENSION_ESCROW_SERVICE"))
        self.assertFalse(
            hasattr(Security, "CSSMOID_APPLE_EXTENSION_PROVISIONING_PROFILE_SIGNING")
        )
        self.assertFalse(hasattr(Security, "CSSMOID_NetscapeCertType"))
        self.assertFalse(hasattr(Security, "CSSMOID_NetscapeCertSequence"))
        self.assertFalse(hasattr(Security, "CSSMOID_NetscapeSGC"))
        self.assertFalse(hasattr(Security, "CSSMOID_MicrosoftSGC"))
        self.assertFalse(hasattr(Security, "CE_NCT_SSL_Client"))
        self.assertFalse(hasattr(Security, "CE_NCT_SSL_Server"))
        self.assertFalse(hasattr(Security, "CE_NCT_SMIME"))
        self.assertFalse(hasattr(Security, "CE_NCT_ObjSign"))
        self.assertFalse(hasattr(Security, "CE_NCT_Reserved"))
        self.assertFalse(hasattr(Security, "CE_NCT_SSL_CA"))
        self.assertFalse(hasattr(Security, "CE_NCT_SMIME_CA"))
        self.assertFalse(hasattr(Security, "CE_NCT_ObjSignCA"))



import Security
from PyObjCTools.TestSupport import TestCase
import objc


class TestAuthSession(TestCase):
    def test_constants(self):
        self.assertEqual(Security.noSecuritySession, 0)
        self.assertEqual(Security.callerSecuritySession, -1)

        self.assertEqual(Security.sessionIsRoot, 0x0001)
        self.assertEqual(Security.sessionHasGraphicAccess, 0x0010)
        self.assertEqual(Security.sessionHasTTY, 0x0020)
        self.assertEqual(Security.sessionIsRemote, 0x1000)

        self.assertEqual(Security.sessionKeepCurrentBootstrap, 0x8000)

        self.assertEqual(Security.errSessionSuccess, 0)
        self.assertEqual(Security.errSessionInvalidId, -60500)
        self.assertEqual(Security.errSessionInvalidAttributes, -60501)
        self.assertEqual(Security.errSessionAuthorizationDenied, -60502)
        self.assertEqual(Security.errSessionValueNotSet, -60503)
        self.assertEqual(Security.errSessionInternal, Security.errAuthorizationInternal)
        self.assertEqual(
            Security.errSessionInvalidFlags, Security.errAuthorizationInvalidFlags
        )

    def test_functions(self):
        self.assertResultHasType(Security.SessionGetInfo, objc._C_INT)
        self.assertArgHasType(Security.SessionGetInfo, 0, objc._C_UINT)
        self.assertArgHasType(
            Security.SessionGetInfo, 1, objc._C_OUT + objc._C_PTR + objc._C_UINT
        )
        self.assertArgHasType(
            Security.SessionGetInfo, 2, objc._C_OUT + objc._C_PTR + objc._C_UINT
        )

        self.assertResultHasType(Security.SessionCreate, objc._C_INT)
        self.assertArgHasType(Security.SessionCreate, 0, objc._C_UINT)
        self.assertArgHasType(Security.SessionCreate, 1, objc._C_UINT)



import Security
from PyObjCTools.TestSupport import TestCase


class TestSecPolicySearch(TestCase):
    def test_unsuppported(self):
        self.assertFalse(hasattr(Security, "SecPolicySearchRef"))
        self.assertFalse(hasattr(Security, "SecPolicySearchGetTypeID"))
        self.assertFalse(hasattr(Security, "SecPolicySearchCreate"))
        self.assertFalse(hasattr(Security, "SecPolicySearchCopyNext"))



import Security
from PyObjCTools.TestSupport import TestCase, min_os_level
import objc


class TestSecAccessControl(TestCase):
    @min_os_level("10.10")
    def test_types(self):
        self.assertIsCFType(Security.SecAccessControlRef)

    def test_constants(self):
        self.assertEqual(Security.kSecAccessControlUserPresence, 1 << 0)
        self.assertEqual(Security.kSecAccessControlBiometryAny, 1 << 1)
        self.assertEqual(Security.kSecAccessControlTouchIDAny, 1 << 1)
        self.assertEqual(Security.kSecAccessControlBiometryCurrentSet, 1 << 3)
        self.assertEqual(Security.kSecAccessControlTouchIDCurrentSet, 1 << 3)
        self.assertEqual(Security.kSecAccessControlDevicePasscode, 1 << 4)
        self.assertEqual(Security.kSecAccessControlOr, 1 << 14)
        self.assertEqual(Security.kSecAccessControlAnd, 1 << 15)
        self.assertEqual(Security.kSecAccessControlPrivateKeyUsage, 1 << 30)
        self.assertEqual(Security.kSecAccessControlApplicationPassword, 1 << 31)
        self.assertEqual(Security.kSecAccessControlDevicePasscode, 1 << 4)
        self.assertEqual(Security.kSecAccessControlWatch, 1 << 5)

    @min_os_level("10.10")
    def test_functions(self):
        self.assertResultHasType(Security.SecAccessControlCreateWithFlags, objc._C_ID)
        self.assertResultIsCFRetained(Security.SecAccessControlCreateWithFlags)
        self.assertArgHasType(Security.SecAccessControlCreateWithFlags, 0, objc._C_ID)
        self.assertArgHasType(Security.SecAccessControlCreateWithFlags, 1, objc._C_ID)
        self.assertArgHasType(
            Security.SecAccessControlCreateWithFlags, 2, objc._C_NSUInteger
        )
        self.assertArgHasType(
            Security.SecAccessControlCreateWithFlags,
            3,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )



import Security
from PyObjCTools.TestSupport import TestCase, min_os_level
import objc

SecTransformActionBlock = b"@"
SecTransformAttributeActionBlock = b"@@@"
SecTransformDataBlock = b"@@"
SecTransformInstanceBlock = b"@"
SecTransformCreateFP = b"@?^{__CFString=}@^{OpaqueSecTransformImplementation=}"


class TestSecCustomTransform(TestCase):
    def test_types(self):
        self.assertIsOpaquePointer(Security.SecTransformImplementationRef)

    def test_constants(self):
        self.assertEqual(Security.kSecTransformMetaAttributeValue, 0)
        self.assertEqual(Security.kSecTransformMetaAttributeName, 1)
        self.assertEqual(Security.kSecTransformMetaAttributeRef, 2)
        self.assertEqual(Security.kSecTransformMetaAttributeRequired, 3)
        self.assertEqual(
            Security.kSecTransformMetaAttributeRequiresOutboundConnection, 4
        )
        self.assertEqual(Security.kSecTransformMetaAttributeDeferred, 5)
        self.assertEqual(Security.kSecTransformMetaAttributeStream, 6)
        self.assertEqual(Security.kSecTransformMetaAttributeCanCycle, 7)
        self.assertEqual(Security.kSecTransformMetaAttributeExternalize, 8)
        self.assertEqual(Security.kSecTransformMetaAttributeHasOutboundConnections, 9)
        self.assertEqual(Security.kSecTransformMetaAttributeHasInboundConnection, 10)

        self.assertIsInstance(Security.kSecTransformActionCanExecute, str)
        self.assertIsInstance(Security.kSecTransformActionStartingExecution, str)
        self.assertIsInstance(Security.kSecTransformActionFinalize, str)
        self.assertIsInstance(Security.kSecTransformActionExternalizeExtraData, str)
        self.assertIsInstance(Security.kSecTransformActionProcessData, str)
        self.assertIsInstance(Security.kSecTransformActionInternalizeExtraData, str)
        self.assertIsInstance(Security.kSecTransformActionAttributeNotification, str)
        self.assertIsInstance(Security.kSecTransformActionAttributeValidation, str)

    def test_functions(self):
        self.assertResultHasType(Security.SecTransformSetAttributeAction, objc._C_ID)
        self.assertArgHasType(
            Security.SecTransformSetAttributeAction,
            0,
            Security.SecTransformImplementationRef.__typestr__,
        )
        self.assertArgHasType(Security.SecTransformSetAttributeAction, 1, objc._C_ID)
        self.assertArgHasType(Security.SecTransformSetAttributeAction, 2, objc._C_ID)
        self.assertArgIsBlock(
            Security.SecTransformSetAttributeAction, 3, SecTransformAttributeActionBlock
        )

        self.assertResultHasType(Security.SecTransformSetDataAction, objc._C_ID)
        self.assertArgHasType(
            Security.SecTransformSetDataAction,
            0,
            Security.SecTransformImplementationRef.__typestr__,
        )
        self.assertArgHasType(Security.SecTransformSetDataAction, 1, objc._C_ID)
        self.assertArgIsBlock(
            Security.SecTransformSetDataAction, 2, SecTransformDataBlock
        )

        self.assertResultHasType(Security.SecTransformSetTransformAction, objc._C_ID)
        self.assertArgHasType(
            Security.SecTransformSetTransformAction,
            0,
            Security.SecTransformImplementationRef.__typestr__,
        )
        self.assertArgHasType(Security.SecTransformSetTransformAction, 1, objc._C_ID)
        self.assertArgIsBlock(
            Security.SecTransformSetTransformAction, 2, SecTransformActionBlock
        )

        self.assertResultHasType(Security.SecTranformCustomGetAttribute, objc._C_ID)
        self.assertArgHasType(
            Security.SecTranformCustomGetAttribute,
            0,
            Security.SecTransformImplementationRef.__typestr__,
        )
        self.assertArgHasType(Security.SecTranformCustomGetAttribute, 1, objc._C_ID)
        self.assertArgHasType(
            Security.SecTranformCustomGetAttribute, 2, objc._C_NSInteger
        )

        self.assertResultHasType(Security.SecTransformCustomGetAttribute, objc._C_ID)
        self.assertArgHasType(
            Security.SecTransformCustomGetAttribute,
            0,
            Security.SecTransformImplementationRef.__typestr__,
        )
        self.assertArgHasType(Security.SecTransformCustomGetAttribute, 1, objc._C_ID)
        self.assertArgHasType(
            Security.SecTransformCustomGetAttribute, 2, objc._C_NSInteger
        )

        self.assertResultHasType(Security.SecTransformCustomSetAttribute, objc._C_ID)
        self.assertArgHasType(
            Security.SecTransformCustomSetAttribute,
            0,
            Security.SecTransformImplementationRef.__typestr__,
        )
        self.assertArgHasType(Security.SecTransformCustomSetAttribute, 1, objc._C_ID)
        self.assertArgHasType(
            Security.SecTransformCustomSetAttribute, 2, objc._C_NSInteger
        )
        self.assertArgHasType(Security.SecTransformCustomSetAttribute, 3, objc._C_ID)

        self.assertResultHasType(Security.SecTransformPushbackAttribute, objc._C_ID)
        self.assertArgHasType(
            Security.SecTransformPushbackAttribute,
            0,
            Security.SecTransformImplementationRef.__typestr__,
        )
        self.assertArgHasType(Security.SecTransformPushbackAttribute, 1, objc._C_ID)
        self.assertArgHasType(Security.SecTransformPushbackAttribute, 2, objc._C_ID)

    @min_os_level("10.7")
    def test_functions_10_7(self):
        self.assertResultHasType(Security.SecTransformRegister, objc._C_NSBOOL)
        self.assertArgHasType(Security.SecTransformRegister, 0, b"^{__CFString=}")
        self.assertArgIsFunction(
            Security.SecTransformRegister, 1, SecTransformCreateFP, 1
        )
        self.assertArgHasType(Security.SecTransformRegister, 2, b"o^^{__CFError=}")

        self.assertResultHasType(Security.SecTransformCreate, objc._C_ID)
        self.assertResultIsCFRetained(Security.SecTransformCreate)
        self.assertArgHasType(Security.SecTransformCreate, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SecTransformCreate, 1, objc._C_OUT + objc._C_PTR + objc._C_ID
        )

        self.assertResultHasType(Security.SecTransformNoData, objc._C_ID)



import Security
from PyObjCTools.TestSupport import TestCase
import objc


class TestSecCodeHost(TestCase):
    def test_constants(self):
        self.assertEqual(Security.kSecCSDedicatedHost, 1 << 0)
        self.assertEqual(Security.kSecCSGenerateGuestHash, 1 << 1)

    def test_functions(self):
        self.assertResultHasType(Security.SecHostCreateGuest, objc._C_INT)
        self.assertArgHasType(Security.SecHostCreateGuest, 0, objc._C_UINT)
        self.assertArgHasType(Security.SecHostCreateGuest, 1, objc._C_UINT)
        self.assertArgHasType(Security.SecHostCreateGuest, 2, objc._C_ID)
        self.assertArgHasType(Security.SecHostCreateGuest, 3, objc._C_ID)
        self.assertArgHasType(Security.SecHostCreateGuest, 4, objc._C_UINT)
        self.assertArgHasType(
            Security.SecHostCreateGuest, 5, objc._C_OUT + objc._C_PTR + objc._C_UINT
        )

        self.assertResultHasType(Security.SecHostRemoveGuest, objc._C_INT)
        self.assertArgHasType(Security.SecHostRemoveGuest, 0, objc._C_UINT)
        self.assertArgHasType(Security.SecHostRemoveGuest, 1, objc._C_UINT)
        self.assertArgHasType(Security.SecHostRemoveGuest, 2, objc._C_UINT)

        self.assertResultHasType(Security.SecHostSelectGuest, objc._C_INT)
        self.assertArgHasType(Security.SecHostSelectGuest, 0, objc._C_UINT)
        self.assertArgHasType(Security.SecHostSelectGuest, 1, objc._C_UINT)

        self.assertResultHasType(Security.SecHostSelectedGuest, objc._C_INT)
        self.assertArgHasType(Security.SecHostSelectedGuest, 0, objc._C_UINT)
        self.assertArgHasType(
            Security.SecHostSelectedGuest, 1, objc._C_OUT + objc._C_PTR + objc._C_UINT
        )

        self.assertResultHasType(Security.SecHostSetGuestStatus, objc._C_INT)
        self.assertArgHasType(Security.SecHostSetGuestStatus, 0, objc._C_UINT)
        self.assertArgHasType(Security.SecHostSetGuestStatus, 1, objc._C_UINT)
        self.assertArgHasType(Security.SecHostSetGuestStatus, 2, objc._C_ID)
        self.assertArgHasType(Security.SecHostSetGuestStatus, 3, objc._C_UINT)

        self.assertResultHasType(Security.SecHostSetHostingPort, objc._C_INT)
        self.assertArgHasType(Security.SecHostSetHostingPort, 0, objc._C_UINT)
        self.assertArgHasType(Security.SecHostSetHostingPort, 1, objc._C_UINT)



import Security
from PyObjCTools.TestSupport import (
    TestCase,
    min_os_level,
    os_release,
    expectedFailureIf,
)
import objc


class TestSecAccess(TestCase):
    def test_constants(self):
        self.assertEqual(Security.kSecUseOnlyUID, 1)
        self.assertEqual(Security.kSecUseOnlyGID, 2)
        self.assertEqual(Security.kSecHonorRoot, 0x100)
        self.assertEqual(
            Security.kSecMatchBits, Security.kSecUseOnlyUID | Security.kSecUseOnlyGID
        )

    @min_os_level("10.7")
    def test_constants_10_7(self):
        self.assertIsInstance(Security.kSecACLAuthorizationAny, str)
        self.assertIsInstance(Security.kSecACLAuthorizationLogin, str)
        self.assertIsInstance(Security.kSecACLAuthorizationGenKey, str)
        self.assertIsInstance(Security.kSecACLAuthorizationDelete, str)
        self.assertIsInstance(Security.kSecACLAuthorizationExportWrapped, str)
        self.assertIsInstance(Security.kSecACLAuthorizationExportClear, str)
        self.assertIsInstance(Security.kSecACLAuthorizationImportWrapped, str)
        self.assertIsInstance(Security.kSecACLAuthorizationImportClear, str)
        self.assertIsInstance(Security.kSecACLAuthorizationSign, str)
        self.assertIsInstance(Security.kSecACLAuthorizationEncrypt, str)
        self.assertIsInstance(Security.kSecACLAuthorizationDecrypt, str)
        self.assertIsInstance(Security.kSecACLAuthorizationMAC, str)
        self.assertIsInstance(Security.kSecACLAuthorizationDerive, str)
        self.assertIsInstance(Security.kSecACLAuthorizationKeychainCreate, str)
        self.assertIsInstance(Security.kSecACLAuthorizationKeychainDelete, str)
        self.assertIsInstance(Security.kSecACLAuthorizationKeychainItemRead, str)
        self.assertIsInstance(Security.kSecACLAuthorizationKeychainItemInsert, str)
        self.assertIsInstance(Security.kSecACLAuthorizationKeychainItemModify, str)
        self.assertIsInstance(Security.kSecACLAuthorizationKeychainItemDelete, str)

    @expectedFailureIf(
        os_release().rsplit(".", 1)[0] in ("10.9", "10.10", "10.11", "10.12")
    )
    @min_os_level("10.13")
    def test_constants_10_13(self):
        self.assertIsInstance(Security.kSecACLAuthorizationChangeACL, str)
        self.assertIsInstance(Security.kSecACLAuthorizationChangeOwner, str)

    @min_os_level("10.11")
    def test_constants_10_11(self):
        self.assertIsInstance(Security.kSecACLAuthorizationPartitionID, str)
        self.assertIsInstance(Security.kSecACLAuthorizationIntegrity, str)

    def test_functions(self):
        self.assertIsInstance(Security.SecAccessGetTypeID(), int)

        self.assertResultHasType(Security.SecAccessCreate, objc._C_INT)
        self.assertArgHasType(Security.SecAccessCreate, 0, objc._C_ID)
        self.assertArgHasType(Security.SecAccessCreate, 1, objc._C_ID)
        self.assertArgHasType(
            Security.SecAccessCreate, 2, objc._C_OUT + objc._C_PTR + objc._C_ID
        )
        self.assertArgIsCFRetained(Security.SecAccessCreate, 2)

        self.assertFalse(hasattr(Security, "SecAccessCreateFromOwnerAndACL"))

        self.assertResultHasType(Security.SecAccessCreateWithOwnerAndACL, objc._C_ID)
        self.assertResultIsCFRetained(Security.SecAccessCreateWithOwnerAndACL)
        self.assertArgHasType(Security.SecAccessCreateWithOwnerAndACL, 0, objc._C_UINT)
        self.assertArgHasType(Security.SecAccessCreateWithOwnerAndACL, 1, objc._C_UINT)
        self.assertArgHasType(Security.SecAccessCreateWithOwnerAndACL, 2, objc._C_UINT)
        self.assertArgHasType(Security.SecAccessCreateWithOwnerAndACL, 3, objc._C_ID)
        self.assertArgHasType(
            Security.SecAccessCreateWithOwnerAndACL,
            4,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )

        self.assertFalse(hasattr(Security, "SecAccessGetOwnerAndACL"))

        self.assertResultHasType(Security.SecAccessCopyOwnerAndACL, objc._C_INT)
        self.assertArgHasType(Security.SecAccessCopyOwnerAndACL, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SecAccessCopyOwnerAndACL,
            1,
            objc._C_OUT + objc._C_PTR + objc._C_UINT,
        )
        self.assertArgHasType(
            Security.SecAccessCopyOwnerAndACL,
            2,
            objc._C_OUT + objc._C_PTR + objc._C_UINT,
        )
        self.assertArgHasType(
            Security.SecAccessCopyOwnerAndACL,
            3,
            objc._C_OUT + objc._C_PTR + objc._C_UINT,
        )
        self.assertArgHasType(
            Security.SecAccessCopyOwnerAndACL, 4, objc._C_OUT + objc._C_PTR + objc._C_ID
        )
        self.assertArgIsCFRetained(Security.SecAccessCopyOwnerAndACL, 4)

        self.assertResultHasType(Security.SecAccessCopyACLList, objc._C_INT)
        self.assertArgHasType(Security.SecAccessCopyACLList, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SecAccessCopyACLList, 1, objc._C_OUT + objc._C_PTR + objc._C_ID
        )
        self.assertArgIsCFRetained(Security.SecAccessCopyACLList, 1)

        self.assertResultHasType(Security.SecAccessCopyMatchingACLList, objc._C_ID)
        self.assertResultIsCFRetained(Security.SecAccessCopyMatchingACLList)
        self.assertArgHasType(Security.SecAccessCopyMatchingACLList, 0, objc._C_ID)
        self.assertArgHasType(Security.SecAccessCopyMatchingACLList, 1, objc._C_ID)



import Security
from PyObjCTools.TestSupport import TestCase


class TestCSSMAPI(TestCase):
    def test_unsuppported(self):
        self.assertFalse(hasattr(Security, "CSSM_Init"))
        self.assertFalse(hasattr(Security, "CSSM_Terminate"))
        self.assertFalse(hasattr(Security, "CSSM_ModuleLoad"))
        self.assertFalse(hasattr(Security, "CSSM_ModuleUnload"))
        self.assertFalse(hasattr(Security, "CSSM_Introduce"))
        self.assertFalse(hasattr(Security, "CSSM_Unintroduce"))
        self.assertFalse(hasattr(Security, "CSSM_ModuleAttach"))
        self.assertFalse(hasattr(Security, "CSSM_ModuleDetach"))
        self.assertFalse(hasattr(Security, "CSSM_SetPrivilege"))
        self.assertFalse(hasattr(Security, "CSSM_GetPrivilege"))
        self.assertFalse(hasattr(Security, "CSSM_GetModuleGUIDFromHandle"))
        self.assertFalse(hasattr(Security, "CSSM_GetSubserviceUIDFromHandle"))
        self.assertFalse(hasattr(Security, "CSSM_ListAttachedModuleManagers"))
        self.assertFalse(hasattr(Security, "CSSM_GetAPIMemoryFunctions"))
        self.assertFalse(hasattr(Security, "CSSM_CSP_CreateSignatureContext"))
        self.assertFalse(hasattr(Security, "CSSM_CSP_CreateSymmetricContext"))
        self.assertFalse(hasattr(Security, "CSSM_CSP_CreateDigestContext"))
        self.assertFalse(hasattr(Security, "CSSM_CSP_CreateMacContext"))
        self.assertFalse(hasattr(Security, "CSSM_CSP_CreateRandomGenContext"))
        self.assertFalse(hasattr(Security, "CSSM_CSP_CreateAsymmetricContext"))
        self.assertFalse(hasattr(Security, "CSSM_CSP_CreateDeriveKeyContext"))
        self.assertFalse(hasattr(Security, "CSSM_CSP_CreateKeyGenContext"))
        self.assertFalse(hasattr(Security, "CSSM_CSP_CreatePassThroughContext"))
        self.assertFalse(hasattr(Security, "CSSM_GetContext"))
        self.assertFalse(hasattr(Security, "CSSM_FreeContext"))
        self.assertFalse(hasattr(Security, "CSSM_SetContext"))
        self.assertFalse(hasattr(Security, "CSSM_DeleteContext"))
        self.assertFalse(hasattr(Security, "CSSM_GetContextAttribute"))
        self.assertFalse(hasattr(Security, "CSSM_UpdateContextAttributes"))
        self.assertFalse(hasattr(Security, "CSSM_DeleteContextAttributes"))
        self.assertFalse(hasattr(Security, "CSSM_CSP_Login"))
        self.assertFalse(hasattr(Security, "CSSM_CSP_Logout"))
        self.assertFalse(hasattr(Security, "CSSM_CSP_GetLoginAcl"))
        self.assertFalse(hasattr(Security, "CSSM_CSP_ChangeLoginAcl"))
        self.assertFalse(hasattr(Security, "CSSM_GetKeyAcl"))
        self.assertFalse(hasattr(Security, "CSSM_ChangeKeyAcl"))
        self.assertFalse(hasattr(Security, "CSSM_GetKeyOwner"))
        self.assertFalse(hasattr(Security, "CSSM_ChangeKeyOwner"))
        self.assertFalse(hasattr(Security, "CSSM_CSP_GetLoginOwner"))
        self.assertFalse(hasattr(Security, "CSSM_CSP_ChangeLoginOwner"))
        self.assertFalse(hasattr(Security, "CSSM_SignData"))
        self.assertFalse(hasattr(Security, "CSSM_SignDataInit"))
        self.assertFalse(hasattr(Security, "CSSM_SignDataUpdate"))
        self.assertFalse(hasattr(Security, "CSSM_SignDataFinal"))
        self.assertFalse(hasattr(Security, "CSSM_VerifyData"))
        self.assertFalse(hasattr(Security, "CSSM_VerifyDataInit"))
        self.assertFalse(hasattr(Security, "CSSM_VerifyDataUpdate"))
        self.assertFalse(hasattr(Security, "CSSM_VerifyDataFinal"))
        self.assertFalse(hasattr(Security, "CSSM_DigestData"))
        self.assertFalse(hasattr(Security, "CSSM_DigestDataInit"))
        self.assertFalse(hasattr(Security, "CSSM_DigestDataUpdate"))
        self.assertFalse(hasattr(Security, "CSSM_DigestDataClone"))
        self.assertFalse(hasattr(Security, "CSSM_DigestDataFinal"))
        self.assertFalse(hasattr(Security, "CSSM_GenerateMac"))
        self.assertFalse(hasattr(Security, "CSSM_GenerateMacInit"))
        self.assertFalse(hasattr(Security, "CSSM_GenerateMacUpdate"))
        self.assertFalse(hasattr(Security, "CSSM_GenerateMacFinal"))
        self.assertFalse(hasattr(Security, "CSSM_VerifyMac"))
        self.assertFalse(hasattr(Security, "CSSM_VerifyMacInit"))
        self.assertFalse(hasattr(Security, "CSSM_VerifyMacUpdate"))
        self.assertFalse(hasattr(Security, "CSSM_VerifyMacFinal"))
        self.assertFalse(hasattr(Security, "CSSM_QuerySize"))
        self.assertFalse(hasattr(Security, "CSSM_EncryptData"))
        self.assertFalse(hasattr(Security, "CSSM_EncryptDataP"))
        self.assertFalse(hasattr(Security, "CSSM_EncryptDataInit"))
        self.assertFalse(hasattr(Security, "CSSM_EncryptDataInitP"))
        self.assertFalse(hasattr(Security, "CSSM_EncryptDataUpdate"))
        self.assertFalse(hasattr(Security, "CSSM_EncryptDataFinal"))
        self.assertFalse(hasattr(Security, "CSSM_DecryptData"))
        self.assertFalse(hasattr(Security, "CSSM_DecryptDataP"))
        self.assertFalse(hasattr(Security, "CSSM_DecryptDataInit"))
        self.assertFalse(hasattr(Security, "CSSM_DecryptDataInitP"))
        self.assertFalse(hasattr(Security, "CSSM_DecryptDataUpdate"))
        self.assertFalse(hasattr(Security, "CSSM_DecryptDataFinal"))
        self.assertFalse(hasattr(Security, "CSSM_QueryKeySizeInBits"))
        self.assertFalse(hasattr(Security, "CSSM_GenerateKey"))
        self.assertFalse(hasattr(Security, "CSSM_GenerateKeyP"))
        self.assertFalse(hasattr(Security, "CSSM_GenerateKeyPair"))
        self.assertFalse(hasattr(Security, "CSSM_GenerateKeyPairP"))
        self.assertFalse(hasattr(Security, "CSSM_GenerateRandom"))
        self.assertFalse(hasattr(Security, "CSSM_CSP_ObtainPrivateKeyFromPublicKey"))
        self.assertFalse(hasattr(Security, "CSSM_WrapKey"))
        self.assertFalse(hasattr(Security, "CSSM_UnwrapKey"))
        self.assertFalse(hasattr(Security, "CSSM_WrapKeyP"))
        self.assertFalse(hasattr(Security, "CSSM_UnwrapKeyP"))
        self.assertFalse(hasattr(Security, "CSSM_DeriveKey"))
        self.assertFalse(hasattr(Security, "CSSM_FreeKey"))
        self.assertFalse(hasattr(Security, "CSSM_GenerateAlgorithmParams"))
        self.assertFalse(hasattr(Security, "CSSM_CSP_GetOperationalStatistics"))
        self.assertFalse(hasattr(Security, "CSSM_GetTimeValue"))
        self.assertFalse(hasattr(Security, "CSSM_RetrieveUniqueId"))
        self.assertFalse(hasattr(Security, "CSSM_RetrieveCounter"))
        self.assertFalse(hasattr(Security, "CSSM_VerifyDevice"))
        self.assertFalse(hasattr(Security, "CSSM_CSP_PassThrough"))
        self.assertFalse(hasattr(Security, "CSSM_TP_SubmitCredRequest"))
        self.assertFalse(hasattr(Security, "CSSM_TP_RetrieveCredResult"))
        self.assertFalse(hasattr(Security, "CSSM_TP_ConfirmCredResult"))
        self.assertFalse(hasattr(Security, "CSSM_TP_ReceiveConfirmation"))
        self.assertFalse(hasattr(Security, "CSSM_TP_CertReclaimKey"))
        self.assertFalse(hasattr(Security, "CSSM_TP_CertReclaimAbort"))
        self.assertFalse(hasattr(Security, "CSSM_TP_FormRequest"))
        self.assertFalse(hasattr(Security, "CSSM_TP_FormSubmit"))
        self.assertFalse(hasattr(Security, "CSSM_TP_CertGroupVerify"))
        self.assertFalse(hasattr(Security, "CSSM_TP_CertCreateTemplate"))
        self.assertFalse(hasattr(Security, "CSSM_TP_CertGetAllTemplateFields"))
        self.assertFalse(hasattr(Security, "CSSM_TP_CertSign"))
        self.assertFalse(hasattr(Security, "CSSM_TP_CrlVerify"))
        self.assertFalse(hasattr(Security, "CSSM_TP_CrlCreateTemplate"))
        self.assertFalse(hasattr(Security, "CSSM_TP_CertRevoke"))
        self.assertFalse(hasattr(Security, "CSSM_TP_CertRemoveFromCrlTemplate"))
        self.assertFalse(hasattr(Security, "CSSM_TP_CrlSign"))
        self.assertFalse(hasattr(Security, "CSSM_TP_ApplyCrlToDb"))
        self.assertFalse(hasattr(Security, "CSSM_TP_CertGroupConstruct"))
        self.assertFalse(hasattr(Security, "CSSM_TP_CertGroupPrune"))
        self.assertFalse(hasattr(Security, "CSSM_TP_CertGroupToTupleGroup"))
        self.assertFalse(hasattr(Security, "CSSM_TP_TupleGroupToCertGroup"))
        self.assertFalse(hasattr(Security, "CSSM_TP_PassThrough"))
        self.assertFalse(hasattr(Security, "CSSM_AC_AuthCompute"))
        self.assertFalse(hasattr(Security, "CSSM_AC_PassThrough"))
        self.assertFalse(hasattr(Security, "CSSM_CL_CertCreateTemplate"))
        self.assertFalse(hasattr(Security, "CSSM_CL_CertGetAllTemplateFields"))
        self.assertFalse(hasattr(Security, "CSSM_CL_CertSign"))
        self.assertFalse(hasattr(Security, "CSSM_CL_CertVerify"))
        self.assertFalse(hasattr(Security, "CSSM_CL_CertVerifyWithKey"))
        self.assertFalse(hasattr(Security, "CSSM_CL_CertGetFirstFieldValue"))
        self.assertFalse(hasattr(Security, "CSSM_CL_CertGetNextFieldValue"))
        self.assertFalse(hasattr(Security, "CSSM_CL_CertAbortQuery"))
        self.assertFalse(hasattr(Security, "CSSM_CL_CertGetKeyInfo"))
        self.assertFalse(hasattr(Security, "CSSM_CL_CertGetAllFields"))
        self.assertFalse(hasattr(Security, "CSSM_CL_FreeFields"))
        self.assertFalse(hasattr(Security, "CSSM_CL_FreeFieldValue"))
        self.assertFalse(hasattr(Security, "CSSM_CL_CertCache"))
        self.assertFalse(hasattr(Security, "CSSM_CL_CertGetFirstCachedFieldValue"))
        self.assertFalse(hasattr(Security, "CSSM_CL_CertGetNextCachedFieldValue"))
        self.assertFalse(hasattr(Security, "CSSM_CL_CertAbortCache"))
        self.assertFalse(hasattr(Security, "CSSM_CL_CertGroupToSignedBundle"))
        self.assertFalse(hasattr(Security, "CSSM_CL_CertGroupFromVerifiedBundle"))
        self.assertFalse(hasattr(Security, "CSSM_CL_CertDescribeFormat"))
        self.assertFalse(hasattr(Security, "CSSM_CL_CrlCreateTemplate"))
        self.assertFalse(hasattr(Security, "CSSM_CL_CrlSetFields"))
        self.assertFalse(hasattr(Security, "CSSM_CL_CrlAddCert"))
        self.assertFalse(hasattr(Security, "CSSM_CL_CertGetKeyInfo"))
        self.assertFalse(hasattr(Security, "CSSM_CL_CertGetAllFields"))
        self.assertFalse(hasattr(Security, "CSSM_CL_FreeFields"))
        self.assertFalse(hasattr(Security, "CSSM_CL_FreeFieldValue"))
        self.assertFalse(hasattr(Security, "CSSM_CL_CertCache"))
        self.assertFalse(hasattr(Security, "CSSM_CL_CertGetFirstCachedFieldValue"))
        self.assertFalse(hasattr(Security, "CSSM_CL_CertGetNextCachedFieldValue"))
        self.assertFalse(hasattr(Security, "CSSM_CL_CertAbortCache"))
        self.assertFalse(hasattr(Security, "CSSM_CL_CertGroupToSignedBundle"))
        self.assertFalse(hasattr(Security, "CSSM_CL_CertGroupFromVerifiedBundle"))
        self.assertFalse(hasattr(Security, "CSSM_CL_CertDescribeFormat"))
        self.assertFalse(hasattr(Security, "CSSM_CL_CrlCreateTemplate"))
        self.assertFalse(hasattr(Security, "CSSM_CL_CrlSetFields"))
        self.assertFalse(hasattr(Security, "CSSM_CL_CrlAddCert"))
        self.assertFalse(hasattr(Security, "CSSM_CL_CrlRemoveCert"))
        self.assertFalse(hasattr(Security, "CSSM_CL_CrlSign"))
        self.assertFalse(hasattr(Security, "CSSM_CL_CrlVerify"))
        self.assertFalse(hasattr(Security, "CSSM_CL_CrlVerifyWithKey"))
        self.assertFalse(hasattr(Security, "CSSM_CL_IsCertInCrl"))
        self.assertFalse(hasattr(Security, "CSSM_CL_CrlGetFirstFieldValue"))
        self.assertFalse(hasattr(Security, "CSSM_CL_CrlGetNextFieldValue"))
        self.assertFalse(hasattr(Security, "CSSM_CL_CrlAbortQuery"))
        self.assertFalse(hasattr(Security, "CSSM_CL_CrlGetAllFields"))
        self.assertFalse(hasattr(Security, "CSSM_CL_CrlCache"))
        self.assertFalse(hasattr(Security, "CSSM_CL_IsCertInCachedCrl"))
        self.assertFalse(hasattr(Security, "CSSM_CL_CrlGetFirstCachedFieldValue"))
        self.assertFalse(hasattr(Security, "CSSM_CL_CrlGetNextCachedFieldValue"))
        self.assertFalse(hasattr(Security, "CSSM_CL_CrlGetAllCachedRecordFields"))
        self.assertFalse(hasattr(Security, "CSSM_CL_CrlAbortCache"))
        self.assertFalse(hasattr(Security, "CSSM_CL_CrlDescribeFormat"))
        self.assertFalse(hasattr(Security, "CSSM_CL_PassThrough"))
        self.assertFalse(hasattr(Security, "CSSM_DL_DbOpen"))
        self.assertFalse(hasattr(Security, "CSSM_DL_DbClose"))
        self.assertFalse(hasattr(Security, "CSSM_DL_DbCreate"))
        self.assertFalse(hasattr(Security, "CSSM_DL_DbDelete"))
        self.assertFalse(hasattr(Security, "CSSM_DL_CreateRelation"))
        self.assertFalse(hasattr(Security, "CSSM_DL_DestroyRelation"))
        self.assertFalse(hasattr(Security, "CSSM_DL_Authenticate"))
        self.assertFalse(hasattr(Security, "CSSM_DL_GetDbAcl"))
        self.assertFalse(hasattr(Security, "CSSM_DL_ChangeDbAcl"))
        self.assertFalse(hasattr(Security, "CSSM_DL_GetDbOwner"))
        self.assertFalse(hasattr(Security, "CSSM_DL_ChangeDbOwner"))
        self.assertFalse(hasattr(Security, "CSSM_DL_GetDbNames"))
        self.assertFalse(hasattr(Security, "CSSM_DL_GetDbNameFromHandle"))
        self.assertFalse(hasattr(Security, "CSSM_DL_FreeNameList"))
        self.assertFalse(hasattr(Security, "CSSM_DL_DataInsert"))
        self.assertFalse(hasattr(Security, "CSSM_DL_DataDelete"))
        self.assertFalse(hasattr(Security, "CSSM_DL_DataModify"))
        self.assertFalse(hasattr(Security, "CSSM_DL_DataGetFirst"))
        self.assertFalse(hasattr(Security, "CSSM_DL_DataGetNext"))
        self.assertFalse(hasattr(Security, "CSSM_DL_DataAbortQuery"))
        self.assertFalse(hasattr(Security, "CSSM_DL_DataGetFromUniqueRecordId"))
        self.assertFalse(hasattr(Security, "CSSM_DL_FreeUniqueRecord"))
        self.assertFalse(hasattr(Security, "CSSM_DL_PassThrough"))



import sys

import Security
from PyObjCTools.TestSupport import TestCase, fourcc
import objc

SecKeychainCallback = (
    b"iI^{SecKeychainCallbackInfo=I^{__SecKeychainItem=}^{__SecKeychain=}i}^v"
)


class TestSecKeychain(TestCase):
    def test_structs(self):
        v = Security.SecKeychainSettings()
        self.assertEqual(v.version, 0)
        self.assertEqual(v.lockOnSleep, False)
        self.assertEqual(v.useLockInterval, False)
        self.assertEqual(v.lockInterval, 0)
        self.assertPickleRoundTrips(v)

        v = Security.SecKeychainCallbackInfo()
        self.assertEqual(v.version, 0)
        self.assertEqual(v.item, None)
        self.assertEqual(v.keychain, None)
        self.assertEqual(v.pid, 0)
        self.assertPickleRoundTrips(v)

    def test_constants(self):
        self.assertEqual(Security.kSecUnlockStateStatus, 1)
        self.assertEqual(Security.kSecReadPermStatus, 2)
        self.assertEqual(Security.kSecWritePermStatus, 4)

        if sys.byteorder == "little":

            def swap_int(v):
                import struct

                return struct.unpack("<I", struct.pack(">I", v))[0]

            self.assertEqual(
                Security.kSecAuthenticationTypeNTLM, swap_int(fourcc(b"ntlm"))
            )
            self.assertEqual(
                Security.kSecAuthenticationTypeMSN, swap_int(fourcc(b"msna"))
            )
            self.assertEqual(
                Security.kSecAuthenticationTypeDPA, swap_int(fourcc(b"dpaa"))
            )
            self.assertEqual(
                Security.kSecAuthenticationTypeRPA, swap_int(fourcc(b"rpaa"))
            )
            self.assertEqual(
                Security.kSecAuthenticationTypeHTTPBasic, swap_int(fourcc(b"http"))
            )
            self.assertEqual(
                Security.kSecAuthenticationTypeHTTPDigest, swap_int(fourcc(b"httd"))
            )
            self.assertEqual(
                Security.kSecAuthenticationTypeHTMLForm, swap_int(fourcc(b"form"))
            )
            self.assertEqual(
                Security.kSecAuthenticationTypeDefault, swap_int(fourcc(b"dflt"))
            )
            self.assertEqual(Security.kSecAuthenticationTypeAny, 0)

        else:
            self.assertEqual(Security.kSecAuthenticationTypeNTLM, fourcc(b"ntlm"))
            self.assertEqual(Security.kSecAuthenticationTypeMSN, fourcc(b"msna"))
            self.assertEqual(Security.kSecAuthenticationTypeDPA, fourcc(b"dpaa"))
            self.assertEqual(Security.kSecAuthenticationTypeRPA, fourcc(b"rpaa"))
            self.assertEqual(Security.kSecAuthenticationTypeHTTPBasic, fourcc(b"http"))
            self.assertEqual(Security.kSecAuthenticationTypeHTTPDigest, fourcc(b"httd"))
            self.assertEqual(Security.kSecAuthenticationTypeHTMLForm, fourcc(b"form"))
            self.assertEqual(Security.kSecAuthenticationTypeDefault, fourcc(b"dflt"))
            self.assertEqual(Security.kSecAuthenticationTypeAny, 0)

        self.assertEqual(Security.kSecProtocolTypeFTP, fourcc(b"ftp "))
        self.assertEqual(Security.kSecProtocolTypeFTPAccount, fourcc(b"ftpa"))
        self.assertEqual(Security.kSecProtocolTypeHTTP, fourcc(b"http"))
        self.assertEqual(Security.kSecProtocolTypeIRC, fourcc(b"irc "))
        self.assertEqual(Security.kSecProtocolTypeNNTP, fourcc(b"nntp"))
        self.assertEqual(Security.kSecProtocolTypePOP3, fourcc(b"pop3"))
        self.assertEqual(Security.kSecProtocolTypeSMTP, fourcc(b"smtp"))
        self.assertEqual(Security.kSecProtocolTypeSOCKS, fourcc(b"sox "))
        self.assertEqual(Security.kSecProtocolTypeIMAP, fourcc(b"imap"))
        self.assertEqual(Security.kSecProtocolTypeLDAP, fourcc(b"ldap"))
        self.assertEqual(Security.kSecProtocolTypeAppleTalk, fourcc(b"atlk"))
        self.assertEqual(Security.kSecProtocolTypeAFP, fourcc(b"afp "))
        self.assertEqual(Security.kSecProtocolTypeTelnet, fourcc(b"teln"))
        self.assertEqual(Security.kSecProtocolTypeSSH, fourcc(b"ssh "))
        self.assertEqual(Security.kSecProtocolTypeFTPS, fourcc(b"ftps"))
        self.assertEqual(Security.kSecProtocolTypeHTTPS, fourcc(b"htps"))
        self.assertEqual(Security.kSecProtocolTypeHTTPProxy, fourcc(b"htpx"))
        self.assertEqual(Security.kSecProtocolTypeHTTPSProxy, fourcc(b"htsx"))
        self.assertEqual(Security.kSecProtocolTypeFTPProxy, fourcc(b"ftpx"))
        self.assertEqual(Security.kSecProtocolTypeCIFS, fourcc(b"cifs"))
        self.assertEqual(Security.kSecProtocolTypeSMB, fourcc(b"smb "))
        self.assertEqual(Security.kSecProtocolTypeRTSP, fourcc(b"rtsp"))
        self.assertEqual(Security.kSecProtocolTypeRTSPProxy, fourcc(b"rtsx"))
        self.assertEqual(Security.kSecProtocolTypeDAAP, fourcc(b"daap"))
        self.assertEqual(Security.kSecProtocolTypeEPPC, fourcc(b"eppc"))
        self.assertEqual(Security.kSecProtocolTypeIPP, fourcc(b"ipp "))
        self.assertEqual(Security.kSecProtocolTypeNNTPS, fourcc(b"ntps"))
        self.assertEqual(Security.kSecProtocolTypeLDAPS, fourcc(b"ldps"))
        self.assertEqual(Security.kSecProtocolTypeTelnetS, fourcc(b"tels"))
        self.assertEqual(Security.kSecProtocolTypeIMAPS, fourcc(b"imps"))
        self.assertEqual(Security.kSecProtocolTypeIRCS, fourcc(b"ircs"))
        self.assertEqual(Security.kSecProtocolTypePOP3S, fourcc(b"pops"))
        self.assertEqual(Security.kSecProtocolTypeCVSpserver, fourcc(b"cvsp"))
        self.assertEqual(Security.kSecProtocolTypeSVN, fourcc(b"svn "))
        self.assertEqual(Security.kSecProtocolTypeAny, 0)

        self.assertEqual(Security.kSecLockEvent, 1)
        self.assertEqual(Security.kSecUnlockEvent, 2)
        self.assertEqual(Security.kSecAddEvent, 3)
        self.assertEqual(Security.kSecDeleteEvent, 4)
        self.assertEqual(Security.kSecUpdateEvent, 5)
        self.assertEqual(Security.kSecPasswordChangedEvent, 6)
        self.assertEqual(Security.kSecDefaultChangedEvent, 9)
        self.assertEqual(Security.kSecDataAccessEvent, 10)
        self.assertEqual(Security.kSecKeychainListChangedEvent, 11)
        self.assertEqual(Security.kSecTrustSettingsChangedEvent, 12)

        self.assertEqual(Security.kSecLockEventMask, 1 << Security.kSecLockEvent)
        self.assertEqual(Security.kSecUnlockEventMask, 1 << Security.kSecUnlockEvent)
        self.assertEqual(Security.kSecAddEventMask, 1 << Security.kSecAddEvent)
        self.assertEqual(Security.kSecDeleteEventMask, 1 << Security.kSecDeleteEvent)
        self.assertEqual(Security.kSecUpdateEventMask, 1 << Security.kSecUpdateEvent)
        self.assertEqual(
            Security.kSecPasswordChangedEventMask,
            1 << Security.kSecPasswordChangedEvent,
        )
        self.assertEqual(
            Security.kSecDefaultChangedEventMask, 1 << Security.kSecDefaultChangedEvent
        )
        self.assertEqual(
            Security.kSecDataAccessEventMask, 1 << Security.kSecDataAccessEvent
        )
        self.assertEqual(
            Security.kSecKeychainListChangedMask,
            1 << Security.kSecKeychainListChangedEvent,
        )
        self.assertEqual(
            Security.kSecTrustSettingsChangedEventMask,
            1 << Security.kSecTrustSettingsChangedEvent,
        )
        self.assertEqual(Security.kSecEveryEventMask, 0xFFFFFFFF)

        self.assertEqual(Security.kSecPreferencesDomainUser, 0)
        self.assertEqual(Security.kSecPreferencesDomainSystem, 1)
        self.assertEqual(Security.kSecPreferencesDomainCommon, 2)
        self.assertEqual(Security.kSecPreferencesDomainDynamic, 3)

    def test_functions(self):
        self.assertIsInstance(Security.SecKeychainGetTypeID(), int)

        self.assertResultHasType(Security.SecKeychainGetVersion, objc._C_INT)
        self.assertArgHasType(
            Security.SecKeychainGetVersion, 0, objc._C_OUT + objc._C_PTR + objc._C_UINT
        )

        self.assertResultHasType(Security.SecKeychainOpen, objc._C_INT)
        self.assertArgHasType(
            Security.SecKeychainOpen, 0, objc._C_IN + objc._C_PTR + objc._C_CHAR_AS_TEXT
        )
        self.assertArgIsNullTerminated(Security.SecKeychainOpen, 0)
        self.assertArgHasType(
            Security.SecKeychainOpen, 1, objc._C_OUT + objc._C_PTR + objc._C_ID
        )
        self.assertArgIsCFRetained(Security.SecKeychainOpen, 1)

        self.assertResultHasType(Security.SecKeychainCreate, objc._C_INT)
        self.assertArgHasType(
            Security.SecKeychainCreate,
            0,
            objc._C_IN + objc._C_PTR + objc._C_CHAR_AS_TEXT,
        )
        self.assertArgIsNullTerminated(Security.SecKeychainCreate, 0)
        self.assertArgHasType(Security.SecKeychainCreate, 1, objc._C_UINT)
        self.assertArgHasType(
            Security.SecKeychainCreate, 2, objc._C_IN + objc._C_PTR + objc._C_VOID
        )
        self.assertArgSizeInArg(Security.SecKeychainCreate, 2, 1)
        self.assertArgHasType(Security.SecKeychainCreate, 3, objc._C_NSBOOL)
        self.assertArgHasType(Security.SecKeychainCreate, 4, objc._C_ID)
        self.assertArgHasType(
            Security.SecKeychainCreate, 5, objc._C_OUT + objc._C_PTR + objc._C_ID
        )
        self.assertArgIsCFRetained(Security.SecKeychainCreate, 5)

        self.assertResultHasType(Security.SecKeychainDelete, objc._C_INT)
        self.assertArgHasType(Security.SecKeychainDelete, 0, objc._C_ID)

        self.assertResultHasType(Security.SecKeychainSetSettings, objc._C_INT)
        self.assertArgHasType(Security.SecKeychainSetSettings, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SecKeychainSetSettings,
            1,
            objc._C_IN + objc._C_PTR + Security.SecKeychainSettings.__typestr__,
        )

        self.assertResultHasType(Security.SecKeychainCopySettings, objc._C_INT)
        self.assertArgHasType(Security.SecKeychainCopySettings, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SecKeychainCopySettings,
            1,
            objc._C_OUT + objc._C_PTR + Security.SecKeychainSettings.__typestr__,
        )

        self.assertResultHasType(Security.SecKeychainUnlock, objc._C_INT)
        self.assertArgHasType(Security.SecKeychainUnlock, 0, objc._C_ID)
        self.assertArgHasType(Security.SecKeychainUnlock, 1, objc._C_UINT)
        self.assertArgHasType(
            Security.SecKeychainUnlock, 2, objc._C_IN + objc._C_PTR + objc._C_VOID
        )
        self.assertArgSizeInArg(Security.SecKeychainUnlock, 2, 1)
        self.assertArgHasType(Security.SecKeychainUnlock, 3, objc._C_NSBOOL)

        self.assertResultHasType(Security.SecKeychainLock, objc._C_INT)
        self.assertArgHasType(Security.SecKeychainLock, 0, objc._C_ID)

        self.assertResultHasType(Security.SecKeychainLockAll, objc._C_INT)

        self.assertResultHasType(Security.SecKeychainCopyDefault, objc._C_INT)
        self.assertArgHasType(
            Security.SecKeychainCopyDefault, 0, objc._C_OUT + objc._C_PTR + objc._C_ID
        )
        self.assertArgIsCFRetained(Security.SecKeychainCopyDefault, 0)

        self.assertResultHasType(Security.SecKeychainSetDefault, objc._C_INT)
        self.assertArgHasType(Security.SecKeychainSetDefault, 0, objc._C_ID)

        self.assertResultHasType(Security.SecKeychainCopySearchList, objc._C_INT)
        self.assertArgHasType(
            Security.SecKeychainCopySearchList,
            0,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )
        self.assertArgIsCFRetained(Security.SecKeychainCopySearchList, 0)

        self.assertResultHasType(Security.SecKeychainSetSearchList, objc._C_INT)
        self.assertArgHasType(Security.SecKeychainSetSearchList, 0, objc._C_ID)

        self.assertResultHasType(Security.SecKeychainCopyDomainDefault, objc._C_INT)
        self.assertArgHasType(Security.SecKeychainCopyDomainDefault, 0, objc._C_INT)
        self.assertArgHasType(
            Security.SecKeychainCopyDomainDefault,
            1,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )
        self.assertArgIsCFRetained(Security.SecKeychainCopyDomainDefault, 1)

        self.assertResultHasType(Security.SecKeychainSetDomainDefault, objc._C_INT)
        self.assertArgHasType(Security.SecKeychainSetDomainDefault, 0, objc._C_INT)
        self.assertArgHasType(Security.SecKeychainSetDomainDefault, 1, objc._C_ID)

        self.assertResultHasType(Security.SecKeychainCopyDomainSearchList, objc._C_INT)
        self.assertArgHasType(Security.SecKeychainCopyDomainSearchList, 0, objc._C_INT)
        self.assertArgHasType(
            Security.SecKeychainCopyDomainSearchList,
            1,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )
        self.assertArgIsCFRetained(Security.SecKeychainCopyDomainSearchList, 1)

        self.assertResultHasType(Security.SecKeychainSetDomainSearchList, objc._C_INT)
        self.assertArgHasType(Security.SecKeychainSetDomainSearchList, 0, objc._C_INT)
        self.assertArgHasType(Security.SecKeychainSetDomainSearchList, 1, objc._C_ID)

        self.assertResultHasType(Security.SecKeychainSetPreferenceDomain, objc._C_INT)
        self.assertArgHasType(Security.SecKeychainSetPreferenceDomain, 0, objc._C_INT)

        self.assertResultHasType(Security.SecKeychainGetPreferenceDomain, objc._C_INT)
        self.assertArgHasType(
            Security.SecKeychainGetPreferenceDomain,
            0,
            objc._C_OUT + objc._C_PTR + objc._C_INT,
        )

        self.assertResultHasType(Security.SecKeychainGetStatus, objc._C_INT)
        self.assertArgHasType(Security.SecKeychainGetStatus, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SecKeychainGetStatus, 1, objc._C_OUT + objc._C_PTR + objc._C_UINT
        )

        self.assertResultHasType(Security.SecKeychainGetPath, objc._C_INT)
        self.assertArgHasType(Security.SecKeychainGetPath, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SecKeychainGetPath, 1, objc._C_INOUT + objc._C_PTR + objc._C_UINT
        )
        self.assertArgHasType(
            Security.SecKeychainGetPath,
            2,
            objc._C_OUT + objc._C_PTR + objc._C_CHAR_AS_TEXT,
        )
        self.assertArgSizeInArg(Security.SecKeychainGetPath, 2, 1)

        self.assertResultHasType(Security.SecKeychainAddCallback, objc._C_INT)
        self.assertArgIsFunction(
            Security.SecKeychainAddCallback, 0, SecKeychainCallback, True
        )
        self.assertArgHasType(Security.SecKeychainAddCallback, 1, objc._C_UINT)
        self.assertArgHasType(
            Security.SecKeychainAddCallback, 2, objc._C_PTR + objc._C_VOID
        )

        self.assertResultHasType(Security.SecKeychainRemoveCallback, objc._C_INT)
        self.assertArgIsFunction(
            Security.SecKeychainRemoveCallback, 0, SecKeychainCallback, True
        )

        self.assertResultHasType(Security.SecKeychainAddInternetPassword, objc._C_INT)
        self.assertArgHasType(Security.SecKeychainAddInternetPassword, 0, objc._C_ID)
        self.assertArgHasType(Security.SecKeychainAddInternetPassword, 1, objc._C_UINT)
        self.assertArgHasType(
            Security.SecKeychainAddInternetPassword,
            2,
            objc._C_IN + objc._C_PTR + objc._C_CHAR_AS_TEXT,
        )
        self.assertArgSizeInArg(Security.SecKeychainAddInternetPassword, 2, 1)
        self.assertArgHasType(Security.SecKeychainAddInternetPassword, 3, objc._C_UINT)
        self.assertArgHasType(
            Security.SecKeychainAddInternetPassword,
            4,
            objc._C_IN + objc._C_PTR + objc._C_CHAR_AS_TEXT,
        )
        self.assertArgSizeInArg(Security.SecKeychainAddInternetPassword, 4, 3)
        self.assertArgHasType(Security.SecKeychainAddInternetPassword, 5, objc._C_UINT)
        self.assertArgHasType(
            Security.SecKeychainAddInternetPassword,
            6,
            objc._C_IN + objc._C_PTR + objc._C_CHAR_AS_TEXT,
        )
        self.assertArgSizeInArg(Security.SecKeychainAddInternetPassword, 6, 5)
        self.assertArgHasType(Security.SecKeychainAddInternetPassword, 7, objc._C_UINT)
        self.assertArgHasType(
            Security.SecKeychainAddInternetPassword,
            8,
            objc._C_IN + objc._C_PTR + objc._C_CHAR_AS_TEXT,
        )
        self.assertArgSizeInArg(Security.SecKeychainAddInternetPassword, 8, 7)
        self.assertArgHasType(Security.SecKeychainAddInternetPassword, 9, objc._C_USHT)
        self.assertArgHasType(Security.SecKeychainAddInternetPassword, 10, objc._C_UINT)
        self.assertArgHasType(Security.SecKeychainAddInternetPassword, 11, objc._C_UINT)
        self.assertArgHasType(Security.SecKeychainAddInternetPassword, 12, objc._C_UINT)
        self.assertArgHasType(
            Security.SecKeychainAddInternetPassword,
            13,
            objc._C_IN + objc._C_PTR + objc._C_VOID,
        )
        self.assertArgSizeInArg(Security.SecKeychainAddInternetPassword, 13, 12)
        self.assertArgHasType(
            Security.SecKeychainAddInternetPassword,
            14,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )
        self.assertArgIsCFRetained(Security.SecKeychainAddInternetPassword, 14)

        self.assertResultHasType(Security.SecKeychainAddGenericPassword, objc._C_INT)
        self.assertArgHasType(Security.SecKeychainAddGenericPassword, 0, objc._C_ID)
        self.assertArgHasType(Security.SecKeychainAddGenericPassword, 1, objc._C_UINT)
        self.assertArgHasType(
            Security.SecKeychainAddGenericPassword,
            2,
            objc._C_IN + objc._C_PTR + objc._C_CHAR_AS_TEXT,
        )
        self.assertArgSizeInArg(Security.SecKeychainAddGenericPassword, 2, 1)
        self.assertArgHasType(Security.SecKeychainAddGenericPassword, 3, objc._C_UINT)
        self.assertArgHasType(
            Security.SecKeychainAddGenericPassword,
            4,
            objc._C_IN + objc._C_PTR + objc._C_CHAR_AS_TEXT,
        )
        self.assertArgSizeInArg(Security.SecKeychainAddGenericPassword, 4, 3)
        self.assertArgHasType(Security.SecKeychainAddGenericPassword, 5, objc._C_UINT)
        self.assertArgHasType(
            Security.SecKeychainAddGenericPassword,
            6,
            objc._C_IN + objc._C_PTR + objc._C_VOID,
        )
        self.assertArgSizeInArg(Security.SecKeychainAddGenericPassword, 6, 5)
        self.assertArgHasType(
            Security.SecKeychainAddGenericPassword,
            7,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )
        self.assertArgIsCFRetained(Security.SecKeychainAddGenericPassword, 7)

        self.assertResultHasType(
            Security.SecKeychainSetUserInteractionAllowed, objc._C_INT
        )
        self.assertArgHasType(
            Security.SecKeychainSetUserInteractionAllowed, 0, objc._C_NSBOOL
        )

        self.assertResultHasType(
            Security.SecKeychainGetUserInteractionAllowed, objc._C_INT
        )
        self.assertArgHasType(
            Security.SecKeychainGetUserInteractionAllowed,
            0,
            objc._C_OUT + objc._C_PTR + objc._C_NSBOOL,
        )

        self.assertResultHasType(Security.SecKeychainCopyAccess, objc._C_INT)
        self.assertArgHasType(Security.SecKeychainCopyAccess, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SecKeychainCopyAccess, 1, objc._C_OUT + objc._C_PTR + objc._C_ID
        )
        self.assertArgIsCFRetained(Security.SecKeychainCopyAccess, 1)

        self.assertResultHasType(Security.SecKeychainSetAccess, objc._C_INT)
        self.assertArgHasType(Security.SecKeychainSetAccess, 0, objc._C_ID)
        self.assertArgHasType(Security.SecKeychainSetAccess, 1, objc._C_ID)

    def test_functions_manual(self):
        # XXX: Testing these in unittests is annoyingly hard, hence the lame test below:
        self.assertFalse(
            isinstance(Security.SecKeychainFindInternetPassword, objc.function)
        )
        self.assertFalse(
            isinstance(Security.SecKeychainFindGenericPassword, objc.function)
        )

        self.assertFalse(hasattr(Security, "SecKeychainAttributeInfoForItemID"))
        self.assertFalse(hasattr(Security, "SecKeychainFreeAttributeInfo"))



import Security
from PyObjCTools.TestSupport import TestCase


class Testx509defs(TestCase):
    def test_unsuppported(self):
        self.assertFalse(hasattr(Security, "BER_TAG_UNKNOWN"))
        self.assertFalse(hasattr(Security, "BER_TAG_BOOLEAN"))
        self.assertFalse(hasattr(Security, "BER_TAG_INTEGER"))
        self.assertFalse(hasattr(Security, "BER_TAG_BIT_STRING"))
        self.assertFalse(hasattr(Security, "BER_TAG_OCTET_STRING"))
        self.assertFalse(hasattr(Security, "BER_TAG_NULL"))
        self.assertFalse(hasattr(Security, "BER_TAG_OID"))
        self.assertFalse(hasattr(Security, "BER_TAG_OBJECT_DESCRIPTOR"))
        self.assertFalse(hasattr(Security, "BER_TAG_EXTERNAL"))
        self.assertFalse(hasattr(Security, "BER_TAG_REAL"))
        self.assertFalse(hasattr(Security, "BER_TAG_ENUMERATED"))
        self.assertFalse(hasattr(Security, "BER_TAG_PKIX_UTF8_STRING"))
        self.assertFalse(hasattr(Security, "BER_TAG_SEQUENCE"))
        self.assertFalse(hasattr(Security, "BER_TAG_SET"))
        self.assertFalse(hasattr(Security, "BER_TAG_NUMERIC_STRING"))
        self.assertFalse(hasattr(Security, "BER_TAG_PRINTABLE_STRING"))
        self.assertFalse(hasattr(Security, "BER_TAG_T61_STRING"))
        self.assertFalse(hasattr(Security, "BER_TAG_TELETEX_STRING"))
        self.assertFalse(hasattr(Security, "BER_TAG_VIDEOTEX_STRING"))
        self.assertFalse(hasattr(Security, "BER_TAG_IA5_STRING"))
        self.assertFalse(hasattr(Security, "BER_TAG_UTC_TIME"))
        self.assertFalse(hasattr(Security, "BER_TAG_GENERALIZED_TIME"))
        self.assertFalse(hasattr(Security, "BER_TAG_GRAPHIC_STRING"))
        self.assertFalse(hasattr(Security, "BER_TAG_ISO646_STRING"))
        self.assertFalse(hasattr(Security, "BER_TAG_GENERAL_STRING"))
        self.assertFalse(hasattr(Security, "BER_TAG_VISIBLE_STRING"))
        self.assertFalse(hasattr(Security, "BER_TAG_PKIX_UNIVERSAL_STRING"))
        self.assertFalse(hasattr(Security, "BER_TAG_PKIX_BMP_STRING"))
        self.assertFalse(hasattr(Security, "CSSM_X509_ALGORITHM_IDENTIFIER"))
        self.assertFalse(hasattr(Security, "CSSM_X509_TYPE_VALUE_PAIR"))
        self.assertFalse(hasattr(Security, "CSSM_X509_RDN"))
        self.assertFalse(hasattr(Security, "CSSM_X509_NAME"))
        self.assertFalse(hasattr(Security, "CSSM_X509_SUBJECT_PUBLIC_KEY_INFO"))
        self.assertFalse(hasattr(Security, "CSSM_X509_TIME"))
        self.assertFalse(hasattr(Security, "CSSM_X509_VALIDITY"))
        self.assertFalse(hasattr(Security, "CSSM_X509_OPTION_PRESENT"))
        self.assertFalse(hasattr(Security, "CSSM_X509_OPTION_NOT_PRESENT"))
        self.assertFalse(hasattr(Security, "CSSM_X509EXT_BASICCONSTRAINTS"))
        self.assertFalse(hasattr(Security, "CSSM_X509EXT_DATA_FORMAT;"))
        self.assertFalse(hasattr(Security, "CSSM_X509EXT_TAGandVALUE"))
        self.assertFalse(hasattr(Security, "CSSM_X509EXT_PAIR"))
        self.assertFalse(hasattr(Security, "CSSM_X509_EXTENSION"))
        self.assertFalse(hasattr(Security, "CSSM_X509_EXTENSIONS"))
        self.assertFalse(hasattr(Security, "CSSM_X509_TBS_CERTIFICATE"))
        self.assertFalse(hasattr(Security, "CSSM_X509_SIGNATURE"))
        self.assertFalse(hasattr(Security, "CSSM_X509_SIGNED_CERTIFICATE"))
        self.assertFalse(hasattr(Security, "CSSM_X509EXT_POLICYQUALIFIERINFO"))
        self.assertFalse(hasattr(Security, "CSSM_X509EXT_POLICYQUALIFIERS"))
        self.assertFalse(hasattr(Security, "CSSM_X509EXT_POLICYINFO"))
        self.assertFalse(hasattr(Security, "CSSM_X509_REVOKED_CERT_ENTRY"))
        self.assertFalse(hasattr(Security, "CSSM_X509_REVOKED_CERT_LIST"))
        self.assertFalse(hasattr(Security, "CSSM_X509_TBS_CERTLIST"))
        self.assertFalse(hasattr(Security, "CSSM_X509_SIGNED_CRL"))



import Security
from PyObjCTools.TestSupport import TestCase
import objc


class TestSecACL(TestCase):
    def test_types(self):
        self.assertIsCFType(Security.SecACLRef)

    def test_constants(self):

        self.assertEqual(Security.kSecKeychainPromptRequirePassphase, 0x0001)
        self.assertEqual(Security.kSecKeychainPromptUnsigned, 0x0010)
        self.assertEqual(Security.kSecKeychainPromptUnsignedAct, 0x0020)
        self.assertEqual(Security.kSecKeychainPromptInvalid, 0x0040)
        self.assertEqual(Security.kSecKeychainPromptInvalidAct, 0x0080)

    def test_functions(self):
        self.assertIsInstance(Security.SecACLGetTypeID(), int)

        self.assertFalse(hasattr(Security, "SecACLCreateFromSimpleContents"))

        self.assertResultHasType(Security.SecACLCreateWithSimpleContents, objc._C_INT)
        self.assertArgHasType(Security.SecACLCreateWithSimpleContents, 0, objc._C_ID)
        self.assertArgHasType(Security.SecACLCreateWithSimpleContents, 1, objc._C_ID)
        self.assertArgHasType(Security.SecACLCreateWithSimpleContents, 2, objc._C_ID)
        self.assertArgHasType(Security.SecACLCreateWithSimpleContents, 3, objc._C_USHT)
        self.assertArgHasType(
            Security.SecACLCreateWithSimpleContents,
            4,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )
        self.assertArgIsCFRetained(Security.SecACLCreateWithSimpleContents, 4)

        self.assertResultHasType(Security.SecACLRemove, objc._C_INT)
        self.assertArgHasType(Security.SecACLRemove, 0, objc._C_ID)

        self.assertFalse(hasattr(Security, "SecACLCopySimpleContents"))

        self.assertResultHasType(Security.SecACLCopyContents, objc._C_INT)
        self.assertArgHasType(Security.SecACLCopyContents, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SecACLCopyContents, 1, objc._C_OUT + objc._C_PTR + objc._C_ID
        )
        self.assertArgIsCFRetained(Security.SecACLCopyContents, 1)
        self.assertArgHasType(
            Security.SecACLCopyContents, 2, objc._C_OUT + objc._C_PTR + objc._C_ID
        )
        self.assertArgIsCFRetained(Security.SecACLCopyContents, 2)
        self.assertArgHasType(
            Security.SecACLCopyContents, 3, objc._C_OUT + objc._C_PTR + objc._C_USHT
        )

        self.assertFalse(hasattr(Security, "SecACLSetSimpleContents"))

        self.assertResultHasType(Security.SecACLSetContents, objc._C_INT)
        self.assertArgHasType(Security.SecACLSetContents, 0, objc._C_ID)
        self.assertArgHasType(Security.SecACLSetContents, 1, objc._C_ID)
        self.assertArgHasType(Security.SecACLSetContents, 2, objc._C_ID)
        self.assertArgHasType(Security.SecACLSetContents, 3, objc._C_USHT)

        self.assertFalse(hasattr(Security, "SecACLGetAuthorizations"))

        self.assertResultHasType(Security.SecACLCopyAuthorizations, objc._C_ID)
        self.assertResultIsCFRetained(Security.SecACLCopyAuthorizations)
        self.assertArgHasType(Security.SecACLCopyAuthorizations, 0, objc._C_ID)

        self.assertFalse(hasattr(Security, "SecACLSetAuthorizations"))

        self.assertResultHasType(Security.SecACLUpdateAuthorizations, objc._C_INT)
        self.assertArgHasType(Security.SecACLUpdateAuthorizations, 0, objc._C_ID)
        self.assertArgHasType(Security.SecACLUpdateAuthorizations, 1, objc._C_ID)



import Security
from PyObjCTools.TestSupport import TestCase
import objc


class TestSecStaticCode(TestCase):
    def test_constants(self):
        self.assertIsInstance(Security.kSecCodeAttributeArchitecture, str)
        self.assertIsInstance(Security.kSecCodeAttributeSubarchitecture, str)
        self.assertIsInstance(Security.kSecCodeAttributeUniversalFileOffset, str)
        self.assertIsInstance(Security.kSecCodeAttributeBundleVersion, str)

        self.assertEqual(Security.kSecCSCheckAllArchitectures, 1 << 0)
        self.assertEqual(Security.kSecCSDoNotValidateExecutable, 1 << 1)
        self.assertEqual(Security.kSecCSDoNotValidateResources, 1 << 2)
        self.assertEqual(
            Security.kSecCSBasicValidateOnly,
            Security.kSecCSDoNotValidateExecutable
            | Security.kSecCSDoNotValidateResources,
        )
        self.assertEqual(Security.kSecCSCheckNestedCode, 1 << 3)
        self.assertEqual(Security.kSecCSStrictValidate, 1 << 4)
        self.assertEqual(Security.kSecCSFullReport, 1 << 5)
        self.assertEqual(
            Security.kSecCSCheckGatekeeperArchitectures,
            (1 << 6) | Security.kSecCSCheckAllArchitectures,
        )
        self.assertEqual(Security.kSecCSRestrictSymlinks, 1 << 7)
        self.assertEqual(Security.kSecCSRestrictToAppLike, 1 << 8)
        self.assertEqual(Security.kSecCSRestrictSidebandData, 1 << 9)
        self.assertEqual(Security.kSecCSUseSoftwareSigningCert, 1 << 10)
        self.assertEqual(Security.kSecCSValidatePEH, 1 << 11)
        self.assertEqual(Security.kSecCSSingleThreaded, 1 << 12)
        self.assertEqual(Security.kSecCSAllowNetworkAccess, 1 << 16)
        self.assertEqual(Security.kSecCSFastExecutableValidation, 1 << 17)

    def test_functions(self):
        self.assertIsInstance(Security.SecStaticCodeGetTypeID(), int)

        self.assertResultHasType(Security.SecStaticCodeCreateWithPath, objc._C_INT)
        self.assertArgHasType(Security.SecStaticCodeCreateWithPath, 0, objc._C_ID)
        self.assertArgHasType(Security.SecStaticCodeCreateWithPath, 1, objc._C_UINT)
        self.assertArgHasType(
            Security.SecStaticCodeCreateWithPath,
            2,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )
        self.assertArgIsCFRetained(Security.SecStaticCodeCreateWithPath, 2)

        self.assertResultHasType(
            Security.SecStaticCodeCreateWithPathAndAttributes, objc._C_INT
        )
        self.assertArgHasType(
            Security.SecStaticCodeCreateWithPathAndAttributes, 0, objc._C_ID
        )
        self.assertArgHasType(
            Security.SecStaticCodeCreateWithPathAndAttributes, 1, objc._C_UINT
        )
        self.assertArgHasType(
            Security.SecStaticCodeCreateWithPathAndAttributes, 2, objc._C_ID
        )
        self.assertArgHasType(
            Security.SecStaticCodeCreateWithPathAndAttributes,
            3,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )
        self.assertArgIsCFRetained(Security.SecStaticCodeCreateWithPathAndAttributes, 3)

        self.assertResultHasType(Security.SecStaticCodeCheckValidity, objc._C_INT)
        self.assertArgHasType(Security.SecStaticCodeCheckValidity, 0, objc._C_ID)
        self.assertArgHasType(Security.SecStaticCodeCheckValidity, 1, objc._C_UINT)
        self.assertArgHasType(Security.SecStaticCodeCheckValidity, 2, objc._C_ID)

        self.assertResultHasType(
            Security.SecStaticCodeCheckValidityWithErrors, objc._C_INT
        )
        self.assertArgHasType(
            Security.SecStaticCodeCheckValidityWithErrors, 0, objc._C_ID
        )
        self.assertArgHasType(
            Security.SecStaticCodeCheckValidityWithErrors, 1, objc._C_UINT
        )
        self.assertArgHasType(
            Security.SecStaticCodeCheckValidityWithErrors, 2, objc._C_ID
        )
        self.assertArgHasType(
            Security.SecStaticCodeCheckValidityWithErrors,
            3,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )
